@@ -3,12 +3,16 @@
 ;;;; This software is part of the SBCL system. See the README file for
 ;;;; more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 (macrolet
-    ((def ((name c-name &key do-not-preserve (stack-delta 0))
+    ((def ((name c-name pseudo-atomic &key do-not-preserve (stack-delta 0))
            move-arg
-           move-result)
+           move-result
+           &optional
+           (float-move 'movaps)
+           (float-size 16)
+           (float-sc 'single-reg))
        `(define-assembly-routine
             (,name (:return-style :none))
             ()
@@ -28,26 +32,26 @@
                                     `(inst ,op ,reg)))))
                      (map-floats (op)
                        `(progn
-                          ,@(loop for i by 16
-                                  for float in
-                                  '(float0-tn float1-tn float2-tn float3-tn
-                                    float4-tn float5-tn float6-tn float7-tn
-                                    float8-tn float9-tn float10-tn float11-tn
-                                    float12-tn float13-tn float14-tn float15-tn)
+                          ,@(loop for i by ,float-size
+                                  for offset below 16
+                                  for float = (make-random-tn :kind :normal
+                                                              :sc (sc-or-lose ',float-sc)
+                                                              :offset offset)
                                   collect
                                   (if (eql op 'pop)
-                                      `(inst movaps ,float (make-ea :qword :base rsp-tn :disp ,i))
-                                      `(inst movaps (make-ea :qword :base rsp-tn :disp ,i) ,float))))))
+                                      `(inst ,',float-move ,float (ea ,i rsp-tn))
+                                      `(inst ,',float-move (ea ,i rsp-tn) ,float))))))
             (inst cld)
             (inst push rbp-tn)
             (inst mov rbp-tn rsp-tn)
-            (inst and rsp-tn (- (* n-word-bytes 2)))
-            (inst sub rsp-tn (* 16 16))
+            (inst and rsp-tn (- ,float-size))
+            (inst sub rsp-tn (* 16 ,float-size))
             (map-floats push)
             (map-registers push)
             ,@move-arg
-            ;; asm routines can always call foreign code with a relative operand
-            (inst call (make-fixup ,c-name :foreign))
+            (pseudo-atomic (:elide-if (not ,pseudo-atomic))
+              ;; asm routines can always call foreign code with a relative operand
+              (inst call (make-fixup ,c-name :foreign)))
             ,@move-result
             (map-registers pop)
             (map-floats pop)
@@ -55,51 +59,56 @@
             (inst pop rbp-tn)
             (inst ret ,stack-delta)))))
 
-  (def (alloc-tramp "alloc")
-    ((inst mov rdi-tn (make-ea :qword :base rbp-tn :disp 16))) ; arg
-    ((inst mov (make-ea :qword :base rbp-tn :disp 16) rax-tn))) ; result
+  (def (alloc-tramp "alloc" nil)
+    ((inst mov rdi-tn (ea 16 rbp-tn))) ; arg
+    ((inst mov (ea 16 rbp-tn) rax-tn))) ; result
 
-  (def (alloc-tramp-r11 "alloc"
+  (def (alloc-tramp-r11 "alloc" nil
                         :do-not-preserve (r11-tn)
                         :stack-delta 8) ;; remove the size parameter
-    ((inst mov rdi-tn (make-ea :qword :base rbp-tn :disp 16))) ; arg
+    ((inst mov rdi-tn (ea 16 rbp-tn))) ; arg
+    ((inst mov r11-tn rax-tn))) ; result
+
+  #+avx2
+  (progn
+    (def (alloc-tramp-avx2 "alloc" nil)
+        ((inst mov rdi-tn (ea 16 rbp-tn)))
+      ((inst mov (ea 16 rbp-tn) rax-tn))
+      vmovaps
+      32
+      avx2-reg)
+    (def (alloc-tramp-r11-avx2 "alloc" nil
+                               :do-not-preserve (r11-tn)
+                               :stack-delta 8) ;; remove the size parameter
+        ((inst mov rdi-tn (ea 16 rbp-tn)))     ; arg
+      ((inst mov r11-tn rax-tn))
+        vmovaps
+        32
+        avx2-reg))
+
+  #+immobile-space
+  (def (alloc-layout "alloc_layout" nil :do-not-preserve (r11-tn))
+    () ; no arg
     ((inst mov r11-tn rax-tn))) ; result
 
   ;; These routines are for the deterministic allocation profiler.
   ;; The C support routine's argument is the return PC
-  (def (enable-alloc-counter "allocation_tracker_counted")
-    ((inst lea rdi-tn (make-ea :qword :base rbp-tn :disp 8))) ; arg
+  (def (enable-alloc-counter "allocation_tracker_counted" t)
+    ((inst lea rdi-tn (ea 8 rbp-tn))) ; arg
     ()) ; result
 
-  (def (enable-sized-alloc-counter "allocation_tracker_sized")
-    ((inst lea rdi-tn (make-ea :qword :base rbp-tn :disp 8))) ; arg
+  (def (enable-sized-alloc-counter "allocation_tracker_sized" t)
+    ((inst lea rdi-tn (ea 8 rbp-tn))) ; arg
     ())) ; result
 
-(define-assembly-routine
-    (#!+immobile-code undefined-tramp-fdefn
-     #!-immobile-code undefined-tramp
-     (:return-style :none)
-     #!+immobile-code
-     (:export undefined-tramp))
+(define-assembly-routine (undefined-tramp (:return-style :none))
     ((:temp rax descriptor-reg rax-offset))
-  #!+immobile-code
-  (progn
-    (inst pop rax) ; gets the address of the fdefn (plus some)
-    (inst sub (reg-in-size rax :dword)
-          ;; Subtract the length of the JMP instruction plus offset to the
-          ;; raw-addr-slot, and add back the lowtag. Voila, a tagged descriptor.
-          (+ 5 (ash fdefn-raw-addr-slot word-shift) (- other-pointer-lowtag))))
-  #!+immobile-code
-  UNDEFINED-TRAMP
-  (inst pop (make-ea :qword :base rbp-tn :disp n-word-bytes))
+  (inst pop (ea n-word-bytes rbp-tn))
   (emit-error-break nil cerror-trap (error-number-or-lose 'undefined-fun-error) (list rax))
-  (inst push (make-ea :qword :base rbp-tn :disp n-word-bytes))
-  (inst jmp
-        (make-ea :qword :base rax
-                        :disp (- (* closure-fun-slot n-word-bytes)
-                                 fun-pointer-lowtag))))
+  (inst push (ea n-word-bytes rbp-tn))
+  (inst jmp (ea (- (* closure-fun-slot n-word-bytes) fun-pointer-lowtag) rax)))
 
-#!-sb-dynamic-core
+#-sb-dynamic-core
 (define-assembly-routine
     (undefined-alien-tramp (:return-style :none))
     ()
@@ -108,31 +117,31 @@
   ;; supposed to return with the address from which this alien call was made
   ;; (a PC within that same function) since C convention does not arrange
   ;; for RBP to hold the new frame pointer prior to making a call.
-  ;; This wouldn't matter much because the only restart availableis to throw
+  ;; This wouldn't matter much because the only restart available is to throw
   ;; to toplevel, so a lost frame is not hugely important, but it's annoying.
   (error-call nil 'undefined-alien-fun-error rbx-tn))
 
-#!+sb-dynamic-core
+#+sb-dynamic-core
 (define-assembly-routine
     (undefined-alien-tramp (:return-style :none))
     ()
   (inst push rax-tn) ; save registers in case we want to see the old values
   (inst push rbx-tn)
   ;; load RAX with the PC after the call site
-  (inst mov rax-tn (make-ea :qword :base rsp-tn :disp 16))
+  (inst mov rax-tn (ea 16 rsp-tn))
   ;; load RBX with the signed 32-bit immediate from the call instruction
-  (inst movsx rbx-tn (make-ea :dword :base rax-tn :disp -4))
+  (inst movsx '(:dword :qword) rbx-tn (ea -4 rax-tn))
   ;; if at [PC-5] we see #x25 then it was a call with 32-bit mem addr
   ;; if ...              #xE8 then ...                32-bit offset
-  (inst cmp (make-ea :byte :base rax-tn :disp -5) #x25)
+  (inst cmp :byte (ea -5 rax-tn) #x25)
   (inst jmp :e ABSOLUTE)
-  (inst cmp (make-ea :byte :base rax-tn :disp -5) #xE8)
+  (inst cmp :byte (ea -5 rax-tn) #xE8)
   (inst jmp :e RELATIVE)
   ;; failing those, assume RBX was valid. ("can't happen")
-  (inst mov rbx-tn (make-ea :qword :base rsp-tn)) ; restore pushed value of RBX
+  (inst mov rbx-tn (ea rsp-tn)) ; restore pushed value of RBX
   (inst jmp trap)
   ABSOLUTE
-  (inst lea rbx-tn (make-ea :qword :base rbx-tn :disp -8))
+  (inst lea rbx-tn (ea -8 rbx-tn))
   (inst jmp TRAP)
   RELATIVE
   (inst add rbx-tn rax-tn)
@@ -150,15 +159,15 @@
 ;;; installed as a globally named function. The fdefn contains a jump opcode
 ;;; to a tiny code component specific to the particular closure.
 ;;; The trampoline is responsible for loading RAX, since named calls don't.
-#!-immobile-code
+#-immobile-code
 (define-assembly-routine
     (closure-tramp (:return-style :none))
     ()
   (loadw rax-tn rax-tn fdefn-fun-slot other-pointer-lowtag)
-  (inst jmp (make-ea-for-object-slot rax-tn closure-fun-slot fun-pointer-lowtag)))
+  (inst jmp (object-slot-ea rax-tn closure-fun-slot fun-pointer-lowtag)))
 
 (define-assembly-routine
     (funcallable-instance-tramp (:return-style :none))
     ()
   (loadw rax-tn rax-tn funcallable-instance-function-slot fun-pointer-lowtag)
-  (inst jmp (make-ea-for-object-slot rax-tn closure-fun-slot fun-pointer-lowtag)))
+  (inst jmp (object-slot-ea rax-tn closure-fun-slot fun-pointer-lowtag)))
