@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!IMPL")
+(in-package "SB-IMPL")
 
 ;;; Heap (for the priority queue)
 
@@ -89,6 +89,7 @@
              (:copier nil))
   (contents nil :type vector   :read-only t)
   (keyfun   nil :type function :read-only t))
+(declaim (freeze-type priority-queue))
 
 (defmethod print-object ((object priority-queue) stream)
   (print-unreadable-object (object stream :type t :identity t)
@@ -132,7 +133,7 @@
              (:conc-name %timer-)
              (:constructor
               make-timer
-              (function &key name (thread sb!thread:*current-thread*)))
+              (function &key name (thread sb-thread:*current-thread*)))
              (:copier nil))
   "Timer type. Do not rely on timers being structs as it may change in
 future versions."
@@ -141,9 +142,10 @@ future versions."
   (expire-time        1   :type (or null real))
   (repeat-interval    nil :type (or null (real 0)))
   (catch-up           nil :type boolean)
-  (thread             nil :type (or sb!thread:thread boolean))
+  (thread             nil :type (or sb-thread:thread boolean))
   (interrupt-function nil :type (or null function))
   (cancel-function    nil :type (or null function)))
+(declaim (freeze-type timer))
 
 (defmethod print-object ((timer timer) stream)
   (let ((name (%timer-name timer)))
@@ -155,7 +157,7 @@ future versions."
           ;; identity
           ))))
 
-(setf (fdocumentation 'make-timer 'function)
+(setf (documentation 'make-timer 'function)
       "Create a timer that runs FUNCTION when triggered.
 
 If a THREAD is supplied, FUNCTION is run in that thread. If THREAD is
@@ -182,17 +184,17 @@ from now. For timers with a repeat interval it returns true."
 
 ;;; The scheduler
 
-(defvar *scheduler-lock* (sb!thread:make-mutex :name "Scheduler lock"))
+(define-load-time-global *scheduler-lock* (sb-thread:make-mutex :name "Scheduler lock"))
 
 (defmacro with-scheduler-lock ((&optional) &body body)
   ;; Don't let the SIGALRM handler mess things up.
-  `(sb!thread::with-system-mutex (*scheduler-lock*)
+  `(sb-thread::with-system-mutex (*scheduler-lock*)
      ,@body))
 
 (defun under-scheduler-lock-p ()
-  (sb!thread:holding-mutex-p *scheduler-lock*))
+  (sb-thread:holding-mutex-p *scheduler-lock*))
 
-(defparameter *schedule* (make-priority-queue :key #'%timer-expire-time))
+(define-load-time-global *schedule* (make-priority-queue :key #'%timer-expire-time))
 
 (defun peek-schedule ()
   (priority-queue-maximum *schedule*))
@@ -203,14 +205,14 @@ from now. For timers with a repeat interval it returns true."
 ;;; real time conversion
 
 (defun delta->real (delta)
-  (floor (* delta internal-time-units-per-second)))
+  (floor (* delta sb-xc:internal-time-units-per-second)))
 
 ;;; Public interface
 
 (defun make-cancellable-interruptor (timer)
   ;; return a list of two functions: one that does the same as
   ;; FUNCTION until the other is called, from when it does nothing.
-  (let ((mutex (sb!thread:make-mutex))
+  (let ((mutex (sb-thread:make-mutex))
         (cancelledp nil)
         (function (if (%timer-repeat-interval timer)
                       (lambda ()
@@ -223,12 +225,12 @@ from now. For timers with a repeat interval it returns true."
        ;; Use WITHOUT-INTERRUPTS for the acquiring lock to avoid
        ;; unblocking deferrables unless it's inevitable.
        (without-interrupts
-         (sb!thread:with-recursive-lock (mutex)
+         (sb-thread:with-recursive-lock (mutex)
            (unless cancelledp
              (allow-with-interrupts
                (funcall function))))))
      (lambda ()
-       (sb!thread:with-recursive-lock (mutex)
+       (sb-thread:with-recursive-lock (mutex)
          (setq cancelledp t))))))
 
 (defun %schedule-timer (timer)
@@ -317,8 +319,8 @@ triggers."
                     (catch-up (%timer-catch-up timer))
                     (thread (%timer-thread timer)))
     (when expire-time
-      (if (and (sb!thread::thread-p thread)
-               (not (sb!thread:thread-alive-p thread)))
+      (if (and (sb-thread::thread-p thread)
+               (not (sb-thread:thread-alive-p thread)))
           (unschedule-timer timer)
           (with-scheduler-lock ()
             ;; Schedule at regular intervals. If TIMER has not finished
@@ -342,13 +344,13 @@ triggers."
 ;;; win32 waitable timers using a timerfd-like portability layer in
 ;;; the runtime.
 
-#!+sb-wtimer
+#+sb-wtimer
 (define-alien-type wtimer
-    #!+win32 system-area-pointer ;HANDLE, but that's not defined yet
-    #!+sunos system-area-pointer ;struct os_wtimer *
-    #!+(or android linux bsd) int)
+    #+win32 system-area-pointer ;HANDLE, but that's not defined yet
+    #+sunos system-area-pointer ;struct os_wtimer *
+    #+(or android linux bsd) int)
 
-#!+sb-wtimer
+#+sb-wtimer
 (progn
   (define-alien-routine "os_create_wtimer" wtimer)
   (define-alien-routine "os_wait_for_wtimer" int (wt wtimer))
@@ -363,12 +365,12 @@ triggers."
   (defvar *timer-thread* nil)
 
   (defun get-waitable-timer ()
-    (assert (under-scheduler-lock-p))
+    (aver (under-scheduler-lock-p))
     (or *waitable-timer-handle*
         (prog1
             (setf *waitable-timer-handle* (os-create-wtimer))
           (setf *timer-thread*
-                (sb!thread:make-thread
+                (sb-thread:make-thread
                  (lambda ()
                    (loop while
                         (or (zerop
@@ -381,8 +383,8 @@ triggers."
   (defun itimer-emulation-deinit ()
     (with-scheduler-lock ()
       (when *timer-thread*
-        (sb!thread:terminate-thread *timer-thread*)
-        (sb!thread:join-thread *timer-thread* :default nil))
+        (sb-thread:terminate-thread *timer-thread*)
+        (sb-thread:join-thread *timer-thread* :default nil))
       (when *waitable-timer-handle*
         (os-close-wtimer *waitable-timer-handle*)
         (setf *waitable-timer-handle* nil))))
@@ -401,25 +403,25 @@ triggers."
   (let ((min-nsec 100000))
     (if (minusp time)
         (values 0 min-nsec)
-        (multiple-value-bind (s u) (floor time internal-time-units-per-second)
-          (setf u (floor (* (/ u internal-time-units-per-second)
+        (multiple-value-bind (s u) (floor time sb-xc:internal-time-units-per-second)
+          (setf u (floor (* (/ u sb-xc:internal-time-units-per-second)
                             #.(expt 10 9))))
           (if (and (= 0 s) (< u min-nsec))
               ;; 0 0 means "shut down the timer" for setitimer
               (values 0 min-nsec)
               (values s u))))))
 
-#!-(or sb-wtimer win32)
+#-(or sb-wtimer win32)
 (progn
   (defun %set-system-timer (sec nsec)
-    (sb!unix:unix-setitimer :real 0 0 sec (ceiling nsec 1000)))
+    (sb-unix:unix-setitimer :real 0 0 sec (ceiling nsec 1000)))
 
   (defun %clear-system-timer ()
-    (sb!unix:unix-setitimer :real 0 0 0 0)))
+    (sb-unix:unix-setitimer :real 0 0 0 0)))
 
 (defun set-system-timer ()
-  (assert (under-scheduler-lock-p))
-  (assert (not *interrupts-enabled*))
+  (aver (under-scheduler-lock-p))
+  (aver (not *interrupts-enabled*))
   (let ((next-timer (peek-schedule)))
     (if next-timer
         (let ((delta (- (%timer-expire-time next-timer)
@@ -432,12 +434,16 @@ triggers."
   (let ((function (%timer-interrupt-function timer))
         (thread (%timer-thread timer)))
     (if (eq t thread)
-        (sb!thread:make-thread function :name (format nil "Timer ~A"
+        (sb-thread:make-thread function :name (format nil "Timer ~A"
                                                       (%timer-name timer)))
-        (let ((thread (or thread sb!thread:*current-thread*)))
+        ;; Don't run the timer directly in the current thread.
+        ;; The signal that interrupt-thread sends is blocked so it'll get queued
+        ;; and processed after exiting from SIGALRM-HANDLER.
+        ;; That way we process all pending signals and release the *SCHEDULER-LOCK*.
+        (let ((thread (or thread sb-thread:*current-thread*)))
           (handler-case
-              (sb!thread:interrupt-thread thread function)
-            (sb!thread:interrupt-thread-error (c)
+              (sb-thread:interrupt-thread thread function)
+            (sb-thread:interrupt-thread-error (c)
               (declare (ignore c))
               (warn "Timer ~S failed to interrupt thread ~S."
                     timer thread)))))))
@@ -459,7 +465,7 @@ triggers."
                    (set-system-timer)
                    (return-from run-expired-timers nil)
                 else
-                do (assert (eq timer (priority-queue-extract-maximum *schedule*)))
+                do (aver (eq timer (priority-queue-extract-maximum *schedule*)))
                    (push timer timers)))
         (run-timers)))))
 
