@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;; Instruction-like macros.
 
@@ -76,8 +76,8 @@
   (once-only ((n-target target)
               (n-source source)
               (n-offset offset))
-    (let ((target-offset #!+little-endian n-offset
-                         #!+big-endian `(+ ,n-offset (1- n-word-bytes))))
+    (let ((target-offset #+little-endian n-offset
+                         #+big-endian `(+ ,n-offset (1- n-word-bytes))))
       `(inst ldrb ,n-target (@ ,n-source ,target-offset)))))
 
 ;;; Macros to handle the fact that our stack pointer isn't actually in
@@ -90,9 +90,9 @@
   "Jump to the lisp lip LIP."
   `(let ((function ,function)
          (lip ,lip))
-     (assert (sc-is lip interior-reg))
+     (aver (sc-is lip interior-reg))
      (inst add lip function
-           (- (ash simple-fun-code-offset word-shift)
+           (- (ash simple-fun-insts-offset word-shift)
               fun-pointer-lowtag))
      (inst br lip)))
 
@@ -103,7 +103,7 @@
      ;; Indicate a single-valued return by clearing all of the status
      ;; flags, or a multiple-valued return by setting all of the status
      ;; flags.
-     (assert (sc-is lip interior-reg))
+     (aver (sc-is lip interior-reg))
      ,@(ecase return-style
          (:single-value '((inst msr :nzcv zr-tn)))
          (:multiple-values '((inst orr tmp-tn zr-tn #xf0000000)
@@ -124,7 +124,7 @@
 
 ;;; Move a stack TN to a register and vice-versa.
 (defun load-stack-offset (reg stack stack-tn)
-  (inst ldr reg (@ stack (load-store-offset (* (tn-offset stack-tn) n-word-bytes)))))
+  (inst ldr reg (@ stack (load-store-offset (tn-byte-offset stack-tn)))))
 
 (defmacro load-stack-tn (reg stack)
   `(let ((reg ,reg)
@@ -134,7 +134,7 @@
         (load-stack-offset reg cfp-tn stack)))))
 
 (defun store-stack-offset (reg stack stack-tn)
-  (let ((offset (* (tn-offset stack-tn) n-word-bytes)))
+  (let ((offset (tn-byte-offset stack-tn)))
     (inst str reg (@ stack (load-store-offset offset)))))
 
 (defmacro store-stack-tn (stack reg)
@@ -173,11 +173,11 @@
 ;;; surround a call to ALLOCATION anyway), and to indicate that the
 ;;; P-A FLAG-TN is also acceptable here.
 
-#!+gencgc
+#+gencgc
 (defun allocation-tramp (alloc-tn size back-label return-in-tmp lip)
   (unless (eq size tmp-tn)
     (inst mov tmp-tn size))
-  (load-inline-constant alloc-tn '(:fixup "alloc_tramp" :foreign) lip)
+  (load-inline-constant alloc-tn '(:fixup alloc-tramp :assembly-routine) lip)
   (inst blr alloc-tn)
   (unless return-in-tmp
     (move alloc-tn tmp-tn))
@@ -210,33 +210,32 @@
               ALIGNED2
               (when ,lowtag
                 (inst add ,result-tn ,result-tn ,lowtag))))
-           #!-gencgc
+           #-gencgc
            (t
             (load-symbol-value ,flag-tn *allocation-pointer*)
             (inst add ,result-tn ,flag-tn ,lowtag)
             (inst add ,flag-tn ,flag-tn (add-sub-immediate ,size))
             (store-symbol-value ,flag-tn *allocation-pointer*))
-           #!+gencgc
+           #+gencgc
            (t
             (let ((alloc (gen-label))
                   (back-from-alloc (gen-label))
                   size)
-              #!-sb-thread
+              #-sb-thread
               (progn
-                (load-inline-constant ,flag-tn '(:fixup "boxed_region" :foreign) ,lip)
-                (inst ldp ,result-tn ,flag-tn (@ ,flag-tn)))
-              #!+sb-thread
+                (load-immediate-word ,flag-tn static-space-start)
+                (inst ldp ,result-tn ,flag-tn (@ ,flag-tn (* 2 n-word-bytes))))
+              #+sb-thread
               (inst ldp ,result-tn ,flag-tn (@ thread-tn
                                                (* n-word-bytes thread-alloc-region-slot)))
               (setf size (add-sub-immediate ,size))
               (inst add ,result-tn ,result-tn size)
               (inst cmp ,result-tn ,flag-tn)
               (inst b :hi ALLOC)
-              #!-sb-thread
-              (progn
-                (load-inline-constant ,flag-tn '(:fixup "boxed_region" :foreign) ,lip)
-                (storew ,result-tn ,flag-tn))
-              #!+sb-thread
+              #-sb-thread
+              (inst str ,result-tn (@ null-tn  (- (+ static-space-start (* 2 n-word-bytes))
+                                                  nil-value)))
+              #+sb-thread
               (storew ,result-tn thread-tn thread-alloc-region-slot)
 
               ;; alloc_tramp uses tmp-tn for returning the result,
@@ -249,7 +248,7 @@
               (when ,lowtag
                 (inst add ,result-tn tmp-tn ,lowtag))
 
-              (assemble (*elsewhere*)
+              (assemble (:elsewhere)
                 (emit-label ALLOC)
                 (allocation-tramp ,result-tn
                                   ,size BACK-FROM-ALLOC
@@ -277,8 +276,7 @@
                    :stack-allocate-p ,stack-allocate-p
                    :lip ,lip)
        (when ,type-code
-         (load-immediate-word ,flag-tn (+ (ash (1- ,size) n-widetag-bits)
-                                          ,type-code))
+         (load-immediate-word ,flag-tn (compute-object-header ,size ,type-code))
          (storew ,flag-tn ,result-tn 0 ,lowtag))
        ,@body)))
 
@@ -294,14 +292,10 @@
       (encode-internal-error-args values)
       (emit-alignment 2))))
 
-(defun error-call (vop error-code &rest values)
-  "Cause an error.  ERROR-CODE is the error to cause."
-  (emit-error-break vop error-trap (error-number-or-lose error-code) values))
-
 (defun generate-error-code (vop error-code &rest values)
   "Generate-Error-Code Error-code Value*
   Emit code for an error with the specified Error-Code and context Values."
-  (assemble (*elsewhere*)
+  (assemble (:elsewhere)
     (let ((start-lab (gen-label)))
       (emit-label start-lab)
       (emit-error-break vop
@@ -313,25 +307,33 @@
 
 ;;;; PSEUDO-ATOMIC
 
+#+sb-safepoint
+(defun emit-safepoint ()
+  (inst ldr zr-tn (@ null-tn
+                     (- (+ gc-safepoint-trap-offset n-word-bytes
+                           other-pointer-lowtag)))))
 
 ;;; handy macro for making sequences look atomic
 (defmacro pseudo-atomic ((flag-tn) &body forms)
+  #+sb-safepoint-strictly
+  `(progn ,@forms (emit-safepoint))
+  #-sb-safepoint-strictly
   `(progn
      (without-scheduling ()
-       #!-sb-thread
+       #-sb-thread
        (store-symbol-value csp-tn *pseudo-atomic-atomic*)
-       #!+sb-thread
+       #+sb-thread
        (inst str (32-bit-reg null-tn)
              (@ thread-tn
                 (* n-word-bytes thread-pseudo-atomic-bits-slot))))
      (assemble ()
        ,@forms)
      (without-scheduling ()
-       #!-sb-thread
+       #-sb-thread
        (progn
          (store-symbol-value null-tn *pseudo-atomic-atomic*)
          (load-symbol-value ,flag-tn *pseudo-atomic-interrupted*))
-       #!+sb-thread
+       #+sb-thread
        (progn
          (inst dmb)
          (inst str (32-bit-reg zr-tn)
@@ -343,7 +345,9 @@
        (let ((not-interrputed (gen-label)))
          (inst cbz ,flag-tn not-interrputed)
          (inst brk pending-interrupt-trap)
-         (emit-label not-interrputed)))))
+         (emit-label not-interrputed))
+       #+sb-safepoint
+       (emit-safepoint))))
 
 ;;;; memory accessor vop generators
 
@@ -354,14 +358,20 @@
              `((:translate ,translate)))
      (:policy :fast-safe)
      (:args (object :scs (descriptor-reg))
-            (index :scs (any-reg)))
+            (index :scs (any-reg immediate)))
      (:arg-types ,type tagged-num)
      (:temporary (:scs (interior-reg)) lip)
      (:results (value :scs ,scs))
      (:result-types ,el-type)
      (:generator 5
-       (inst add lip object (lsl index (- word-shift n-fixnum-tag-bits)))
-       (loadw value lip ,offset ,lowtag))))
+       (sc-case index
+         (immediate
+          (inst ldr value (@ object (load-store-offset
+                                     (- (ash (+ ,offset (tn-value index)) word-shift)
+                                        ,lowtag)))))
+         (t
+          (inst add lip object (lsl index (- word-shift n-fixnum-tag-bits)))
+          (loadw value lip ,offset ,lowtag))))))
 
 (defmacro define-full-setter (name type offset lowtag scs el-type
                               &optional translate)
@@ -370,15 +380,21 @@
              `((:translate ,translate)))
      (:policy :fast-safe)
      (:args (object :scs (descriptor-reg))
-            (index :scs (any-reg))
+            (index :scs (any-reg immediate))
             (value :scs ,scs :target result))
      (:arg-types ,type tagged-num ,el-type)
      (:temporary (:scs (interior-reg)) lip)
      (:results (result :scs ,scs))
      (:result-types ,el-type)
      (:generator 2
-       (inst add lip object (lsl index (- word-shift n-fixnum-tag-bits)))
-       (storew value lip ,offset ,lowtag)
+       (sc-case index
+         (immediate
+          (inst str value (@ object (load-store-offset
+                                     (- (ash (+ ,offset (tn-value index)) word-shift)
+                                        ,lowtag)))))
+         (t
+          (inst add lip object (lsl index (- word-shift n-fixnum-tag-bits)))
+          (storew value lip ,offset ,lowtag)))
        (move result value))))
 
 (defmacro define-partial-reffer (name type size signed offset lowtag scs
@@ -388,25 +404,40 @@
              `((:translate ,translate)))
      (:policy :fast-safe)
      (:args (object :scs (descriptor-reg))
-            (index :scs (unsigned-reg)))
-     (:arg-types ,type positive-fixnum)
+            (index :scs (any-reg unsigned-reg immediate)))
+     (:arg-types ,type tagged-num)
      (:results (value :scs ,scs))
      (:result-types ,el-type)
      (:temporary (:scs (interior-reg)) lip)
      (:generator 5
-       ,@(ecase size
-           (:byte
-            `((inst add lip object index)
-              (inst ,(if signed 'ldrsb 'ldrb)
-                    value (@ lip (- (* ,offset n-word-bytes) ,lowtag)))))
-           (:short
-            `((inst add lip object (lsl index 1))
-              (inst ,(if signed 'ldrsh 'ldrh)
-                    value (@ lip (- (* ,offset n-word-bytes) ,lowtag)))))
-           (:word
-            `((inst add lip object (lsl index 2))
-              (inst ,(if signed 'ldrsw 'ldr) (32-bit-reg value)
-                    (@ lip (- (* ,offset n-word-bytes) ,lowtag)))))))))
+       ,@(multiple-value-bind (op shift)
+             (ecase size
+               (:byte
+                (values (if signed 'ldrsb 'ldrb) 0))
+               (:short
+                (values (if signed 'ldrsh 'ldrh) 1))
+               (:word
+                (values (if signed 'ldrsw 'ldr) 2)))
+           (let ((value (if (and (eq size :word)
+                                 (not signed))
+                            '(32-bit-reg value)
+                            'value)))
+             `((sc-case index
+                 (immediate
+                  (inst ,op ,value (@ object (load-store-offset
+                                              (+
+                                               (ash (tn-value index) ,shift)
+                                               (- (* ,offset n-word-bytes) ,lowtag))))))
+                 (t
+                  (let ((shift ,shift))
+                    (sc-case index
+                      (any-reg
+                       (decf shift n-fixnum-tag-bits)))
+                    (inst add lip object (if (minusp shift)
+                                             (asr index (- shift))
+                                             (lsl index shift)))
+                    (inst ,op
+                          ,value (@ lip (- (* ,offset n-word-bytes) ,lowtag))))))))))))
 
 (defmacro define-partial-setter (name type size offset lowtag scs el-type
                                  &optional translate)
@@ -415,23 +446,40 @@
              `((:translate ,translate)))
      (:policy :fast-safe)
      (:args (object :scs (descriptor-reg))
-            (index :scs (unsigned-reg))
+            (index :scs (any-reg unsigned-reg immediate))
             (value :scs ,scs :target result))
-     (:arg-types ,type positive-fixnum ,el-type)
+     (:arg-types ,type tagged-num ,el-type)
      (:temporary (:scs (interior-reg)) lip)
      (:results (result :scs ,scs))
      (:result-types ,el-type)
      (:generator 5
-       ,@(ecase size
-           (:byte
-            `((inst add lip object index)
-              (inst strb value (@ lip (- (* ,offset n-word-bytes) ,lowtag)))))
-           (:short
-            `((inst add lip object (lsl index 1))
-              (inst strh value (@ lip (- (* ,offset n-word-bytes) ,lowtag)))))
-           (:word
-            `((inst add lip object (lsl index 2))
-              (inst str (32-bit-reg value) (@ lip (- (* ,offset n-word-bytes) ,lowtag))))))
+       ,@(multiple-value-bind (op shift)
+             (ecase size
+               (:byte
+                (values 'strb 0))
+               (:short
+                (values 'strh 1))
+               (:word
+                (values 'str 2)))
+           (let ((value (if (eq size :word)
+                            '(32-bit-reg value)
+                            'value)))
+             `((sc-case index
+                 (immediate
+                  (inst ,op ,value (@ object (load-store-offset
+                                              (+
+                                               (ash (tn-value index) ,shift)
+                                               (- (* ,offset n-word-bytes) ,lowtag))))))
+                 (t
+                  (let ((shift ,shift))
+                    (sc-case index
+                      (any-reg
+                       (decf shift n-fixnum-tag-bits)))
+                    (inst add lip object (if (minusp shift)
+                                             (asr index (- shift))
+                                             (lsl index shift)))
+                    (inst ,op
+                          ,value (@ lip (- (* ,offset n-word-bytes) ,lowtag)))))))))
        (move result value))))
 
 (defun load-inline-constant (dst value &optional lip)
@@ -443,31 +491,31 @@
 ;;;
 
 (defmacro load-binding-stack-pointer (reg)
-  #!+sb-thread `(loadw ,reg thread-tn thread-binding-stack-pointer-slot)
-  #!-sb-thread `(load-symbol-value ,reg *binding-stack-pointer*))
+  #+sb-thread `(loadw ,reg thread-tn thread-binding-stack-pointer-slot)
+  #-sb-thread `(load-symbol-value ,reg *binding-stack-pointer*))
 
 (defmacro store-binding-stack-pointer (reg)
-  #!+sb-thread `(storew ,reg thread-tn thread-binding-stack-pointer-slot)
-  #!-sb-thread `(store-symbol-value ,reg *binding-stack-pointer*))
+  #+sb-thread `(storew ,reg thread-tn thread-binding-stack-pointer-slot)
+  #-sb-thread `(store-symbol-value ,reg *binding-stack-pointer*))
 
-#!+sb-thread
+#+sb-thread
 (defmacro tls-index-of (sym)
-  `(@ ,sym (- #!+little-endian 4 other-pointer-lowtag)))
+  `(@ ,sym (- #+little-endian 4 other-pointer-lowtag)))
 
 (defmacro load-tl-symbol-value (reg symbol)
-  #!+sb-thread
+  #+sb-thread
   `(let ((reg ,reg))
      (load-symbol tmp-tn ',symbol)
-     (inst ldr tmp-tn (tls-index-of tmp-tn))
+     (inst ldr (32-bit-reg tmp-tn) (tls-index-of tmp-tn))
      (inst ldr reg (@ thread-tn tmp-tn)))
-  #!-sb-thread
+  #-sb-thread
   `(load-symbol-value ,reg ,symbol))
 
 (defmacro store-tl-symbol-value (reg symbol)
-  #!+sb-thread
+  #+sb-thread
   `(let ((reg ,reg))
      (load-symbol tmp-tn ',symbol)
-     (inst ldr tmp-tn (tls-index-of tmp-tn))
+     (inst ldr (32-bit-reg tmp-tn) (tls-index-of tmp-tn))
      (inst str reg (@ thread-tn tmp-tn)))
-  #!-sb-thread
+  #-sb-thread
   `(store-symbol-value ,reg ,symbol))
