@@ -435,7 +435,7 @@
       (unless (and (typep form '(cons (member list vector)))
                    (do ((items (cdr form))
                         (length 0 (1+ length))
-                        (fun (let ((axis (the (mod #.array-rank-limit) (1+ axis))))
+                        (fun (let ((axis (the (mod #.sb-xc:array-rank-limit) (1+ axis))))
                                (if (= axis rank)
                                    (lambda (item) (push item output))
                                    (lambda (item) (recurse item axis))))))
@@ -716,6 +716,14 @@
                                         ;; we want to avoid reading it.
                                         (the index ,(or c-length 'length))
                                         ,n-words-form))))
+    (when (and c-length
+               fill-pointer
+               (csubtypep (lvar-type fill-pointer) (specifier-type 'index))
+               (not (types-equal-or-intersect (lvar-type fill-pointer)
+                                              (specifier-type `(integer 0 ,c-length)))))
+      (abort-ir1-transform "Invalid fill-pointer ~s for a vector of length ~s."
+                           (type-specifier (lvar-type fill-pointer))
+                           c-length))
     (flet ((eliminate-keywords ()
              (eliminate-keyword-args
               call 1
@@ -725,15 +733,6 @@
                 (:adjustable adjustable)
                 (:fill-pointer fill-pointer))))
            (with-alloc-form (&optional data-wrapper)
-             (when (and c-length
-                        fill-pointer
-                        (csubtypep (lvar-type fill-pointer) (specifier-type 'index))
-                        (not (types-equal-or-intersect (lvar-type fill-pointer)
-                                                       (specifier-type `(integer 0 ,c-length)))))
-               (compiler-warn "Invalid fill-pointer ~s for a vector of length ~s."
-                              (type-specifier (lvar-type fill-pointer))
-                              c-length)
-               (give-up-ir1-transform))
              (cond (complex
                     (let* ((constant-fill-pointer-p (constant-lvar-p fill-pointer))
                            (fill-pointer-value (and constant-fill-pointer-p
@@ -1043,7 +1042,7 @@
                 (not (proper-list-p dims)))
         (give-up-ir1-transform))
       (unless (check-array-dimensions dims call)
-          (give-up-ir1-transform))
+        (give-up-ir1-transform))
       (cond ((singleton-p dims)
              (transform-make-array-vector (car dims) element-type
                                           initial-element initial-contents call
@@ -1420,14 +1419,17 @@
               (let ((index-leaf (ref-leaf index-ref)))
                 (loop for constraint in (ref-constraints array-ref)
                       for y = (constraint-y constraint)
-                      thereis (if (constant-p index-leaf)
-                                  (and (constant-p y)
-                                       (<= (constant-value index-leaf)
-                                           (constant-value y)))
-                                  (eq index-leaf y))))
+                      thereis (and
+                               (eq (constraint-kind constraint) 'array-in-bounds-p)
+                               (if (constant-p index-leaf)
+                                   (and (constant-p y)
+                                        (<= (constant-value index-leaf)
+                                            (constant-value y)))
+                                   (eq index-leaf y)))))
               (loop for constraint in (ref-constraints index-ref)
-                    thereis (eq (constraint-y constraint)
-                                (ref-leaf array-ref)))))
+                    thereis (and (eq (constraint-kind constraint) 'array-in-bounds-p)
+                                 (eq (constraint-y constraint)
+                                     (ref-leaf array-ref))))))
       (give-up-ir1-transform)))
   ;; It's in bounds but it may be of the wrong type
   `(the (and fixnum unsigned-byte) index))
@@ -1871,3 +1873,8 @@
     ((array) node gen)
   (declare (ignore gen))
   (values array (specifier-type '(and array (not (simple-array * (*)))))))
+
+(defoptimizer (%array-fill-pointer-p constraint-propagate-if)
+    ((array) node gen)
+  (declare (ignore gen))
+  (values array (specifier-type '(and vector (not simple-array)))))
