@@ -51,25 +51,6 @@
            (or (not (lexenv-parent lexenv))
                (null-lexenv-p (lexenv-parent lexenv))))))
 
-;;; Return the value to place in %SIMPLE-FUN-INFO given any or all of
-;;; the three possible things that can be stashed there:
-;;;  - string        = just a docstring
-;;;  - simple-vector = just xrefs
-;;;  - cons          = either both of the above, or just a lambda expression
-;;;  - fun-src       = any other combination of the three pieces of data
-(defun pack-simple-fun-info (form doc xrefs)
-  (if form
-      ;; If form starts with a string, we can't store it by itself
-      ;; because it's confusable with (CONS STRING *)
-      ;; Lambda expressions start with LAMBDA, obviously,
-      ;; so this really shouldn't happen. Just being defensive here.
-      (if (or doc xrefs (typep form '(cons string)))
-          (make-fun-src form doc xrefs)
-          form)
-      (if (and doc xrefs)
-          (cons doc xrefs)
-          (or doc xrefs))))
-
 ;;; Initialize INFO structure to correspond to the XEP LAMBDA FUN.
 (defun compute-entry-info (fun info)
   (declare (type clambda fun) (type entry-info info))
@@ -82,12 +63,11 @@
     (setf (entry-info-offset info) (gen-label))
     (setf (entry-info-name info)
           (leaf-debug-name internal-fun))
-    (let ((form (functional-inline-expansion internal-fun))
-          (doc (functional-documentation internal-fun))
-          (xrefs (pack-xref-data (functional-xref internal-fun))))
-      (setf (entry-info-form/doc/xrefs info)
-            (pack-simple-fun-info
-                  (if (fasl-output-p *compile-object*)
+    (setf (entry-info-xref info) (pack-xref-data (functional-xref internal-fun)))
+    (let* ((inline-expansion (functional-inline-expansion internal-fun))
+           (form  (if (fasl-output-p *compile-object*)
+                      ;; If compiling to a file, we only store sources if the STORE-SOURCE
+                      ;; quality value is 3. If to memory, any nonzero value will do.
                       (and (policy bind (= store-source-form 3))
                            ;; Downgrade the error to a warning if this was signaled
                            ;; by SB-PCL::DONT-KNOW-HOW-TO-DUMP.
@@ -103,20 +83,25 @@
                                      ;; on MAKE-LOAD-FORM?  Why did we choose to further obfuscate
                                      ;; a condition that was reflectable and instead turn it
                                      ;; into a dumb text string?
-                                     (when (and (typep c 'simple-error)
-                                                (search "know how to dump"
-                                                        (simple-condition-format-control c)))
-                                       ;; This might be worth a full warning. Dunno.
-                                       ;; After all, the user asked to do what can't be done.
-                                       (compiler-style-warn
+                                     (if (and (typep c 'simple-error)
+                                              (stringp (simple-condition-format-control c))
+                                              (search "know how to dump"
+                                                      (simple-condition-format-control c)))
+                                         ;; This might be worth a full warning. Dunno.
+                                         ;; After all, the user asked to do what can't be done.
+                                         (compiler-style-warn
                                         "Can't preserve function source - ~
 missing MAKE-LOAD-FORM methods?")
-                                       (return nil)))))
-                               (constant-value (find-constant form)))))
+                                         (compiler-style-warn
+                                          "Can't preserve function source: ~A"
+                                          (princ-to-string c)))
+                                       (return nil))))
+                               (constant-value (find-constant inline-expansion)))))
                       (and (policy bind (> store-source-form 0))
-                           form))
-                  doc
-                  xrefs)))
+                           inline-expansion)))
+           (doc (functional-documentation internal-fun)))
+      (setf (entry-info-form/doc info)
+            (if (and form doc) (cons form doc) (or form doc))))
     (when (policy bind (>= debug 1))
       (let ((args (functional-arg-documentation internal-fun)))
         ;; When the component is dumped, the arglists of the entry
