@@ -9,10 +9,10 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
-(defconstant arg-count-sc (make-sc-offset immediate-arg-scn nargs-offset))
-(defconstant closure-sc (make-sc-offset descriptor-reg-sc-number lexenv-offset))
+(defconstant arg-count-sc (make-sc+offset immediate-arg-scn nargs-offset))
+(defconstant closure-sc (make-sc+offset descriptor-reg-sc-number lexenv-offset))
 
 ;;; Make a passing location TN for a local call return PC.  If standard is
 ;;; true, then use the standard (full call) location, otherwise use any legal
@@ -28,10 +28,11 @@
 ;;; standard convention, but is totally unrestricted in non-standard
 ;;; conventions, since we can always fetch it off of the stack using
 ;;; the arg pointer.
-(defun make-old-fp-passing-location (standard)
-  (if standard
-      (make-wired-tn *fixnum-primitive-type* immediate-arg-scn ocfp-offset)
-      (make-normal-tn *fixnum-primitive-type*)))
+(defun make-old-fp-passing-location ()
+  (make-wired-tn *fixnum-primitive-type* immediate-arg-scn ocfp-offset))
+
+(defconstant old-fp-passing-offset
+  (make-sc+offset descriptor-reg-sc-number ocfp-offset))
 
 ;;; Make the TNs used to hold OLD-FP and RETURN-PC within the current
 ;;; function. We treat these specially so that the debugger can find
@@ -118,8 +119,7 @@
     (emit-label start-lab)
     ;; Allocate function header.
     (inst simple-fun-header-word)
-    (dotimes (i (1- simple-fun-code-offset))
-      (inst word 0))
+    (inst .skip (* (1- simple-fun-insts-offset) n-word-bytes))
     ;; The start of the actual code.
     ;; Compute CODE from the address of this entry point.
     (let ((entry-point (gen-label)))
@@ -244,7 +244,7 @@ default-value-8
         ;; gets confused.
         (without-scheduling ()
           (note-this-location vop :single-value-return)
-          (move csp-tn ocfp-tn t)
+          (emit-nop-or-move csp-tn ocfp-tn)
           (inst nop))
         (when lra-label
           (inst compute-code-from-lra code-tn code-tn lra-label temp)))
@@ -259,7 +259,7 @@ default-value-8
           ;; If there are no stack results, clear the stack now.
           (if (> nvals register-arg-count)
               (inst addu temp nargs-tn (fixnumize (- register-arg-count)))
-              (move csp-tn ocfp-tn t)))
+              (emit-nop-or-move csp-tn ocfp-tn)))
 
         ;; Do the single value case.
         (do ((i 1 (1+ i))
@@ -268,7 +268,7 @@ default-value-8
           (move (tn-ref-tn val) null-tn))
         (when (> nvals register-arg-count)
           (inst b default-stack-vals)
-          (move ocfp-tn csp-tn t))
+          (emit-nop-or-move ocfp-tn csp-tn))
 
         (emit-label regs-defaulted)
 
@@ -297,7 +297,7 @@ default-value-8
 
             (let ((defaults (defaults)))
               (aver defaults)
-              (assemble (*elsewhere*)
+              (assemble (:elsewhere)
                 (emit-label default-stack-vals)
                 (do ((remaining defaults (cdr remaining)))
                     ((null remaining))
@@ -348,7 +348,7 @@ default-value-8
 
     (emit-label done)
 
-    (assemble (*elsewhere*)
+    (assemble (:elsewhere)
       (emit-label variable-values)
       (when lra-label
         (inst compute-code-from-lra code-tn code-tn lra-label temp))
@@ -358,7 +358,7 @@ default-value-8
         (storew (first arg) args i))
       (move start args)
       (inst b done)
-      (move count nargs t)))
+      (emit-nop-or-move count nargs)))
   (values))
 
 
@@ -552,7 +552,7 @@ default-value-8
               (bytes-needed-for-non-descriptor-stack-frame))))
     (inst addu lip return-pc-temp (- n-word-bytes other-pointer-lowtag))
     (inst j lip)
-    (move cfp-tn ocfp-temp t)))
+    (emit-nop-or-move cfp-tn ocfp-temp)))
 
 
 ;;;; Full call:
@@ -603,10 +603,13 @@ default-value-8
      (:args
       ,@(unless (eq return :tail)
           '((new-fp :scs (any-reg) :to :eval)))
+      ,@(case named
+          ((nil)
+           '((arg-fun :target lexenv)))
+          (:direct)
+          (t
+           '((name :target name-pass))))
 
-      ,(if named
-           '(name :target name-pass)
-           '(arg-fun :target lexenv))
 
       ,@(when (eq return :tail)
           '((ocfp :target ocfp-pass)
@@ -625,6 +628,7 @@ default-value-8
      (:vop-var vop)
      (:info ,@(unless (or variable (eq return :tail)) '(arg-locs))
             ,@(unless variable '(nargs))
+            ,@(when (eq named :direct) '(fun))
             ,@(when (eq return :fixed) '(nvals))
             step-instrumenting)
 
@@ -644,18 +648,19 @@ default-value-8
                   :to :eval)
                  return-pc-pass)
 
-     ,@(if named
-         `((:temporary (:sc descriptor-reg :offset fdefn-offset
-                        :from (:argument ,(if (eq return :tail) 0 1))
-                        :to :eval)
-                       name-pass))
-
-         `((:temporary (:sc descriptor-reg :offset lexenv-offset
-                        :from (:argument ,(if (eq return :tail) 0 1))
-                        :to :eval)
-                       lexenv)
-           (:temporary (:scs (descriptor-reg) :from (:argument 0) :to :eval)
-                       function)))
+     ,@(case named
+         ((t)
+          `((:temporary (:sc descriptor-reg :offset fdefn-offset
+                         :from (:argument ,(if (eq return :tail) 0 1))
+                         :to :eval)
+                        name-pass)))
+         ((nil)
+          `((:temporary (:sc descriptor-reg :offset lexenv-offset
+                         :from (:argument ,(if (eq return :tail) 0 1))
+                         :to :eval)
+                        lexenv)
+            (:temporary (:scs (descriptor-reg) :from (:argument 0) :to :eval)
+                        function))))
 
      (:temporary (:sc any-reg :offset nargs-offset :to :eval)
                  nargs-pass)
@@ -721,7 +726,7 @@ default-value-8
                             '((:load-ocfp
                                (sc-case ocfp
                                  (any-reg
-                                  (move ocfp-pass ocfp t))
+                                  (emit-nop-or-move ocfp-pass ocfp))
                                  (control-stack
                                   (inst lw ocfp-pass cfp-tn
                                         (ash (tn-offset ocfp)
@@ -729,7 +734,7 @@ default-value-8
                               (:load-return-pc
                                (sc-case return-pc
                                  (descriptor-reg
-                                  (move return-pc-pass return-pc t))
+                                  (emit-nop-or-move return-pc-pass return-pc))
                                  (control-stack
                                   (inst lw return-pc-pass cfp-tn
                                         (ash (tn-offset return-pc)
@@ -743,7 +748,7 @@ default-value-8
                               (:frob-nfp
                                (store-stack-tn nfp-save cur-nfp))
                               (:save-fp
-                               (move ocfp-pass cfp-tn t))
+                               (emit-nop-or-move ocfp-pass cfp-tn))
                               (:load-fp
                                ,(if variable
                                     '(move cfp-tn new-fp)
@@ -755,63 +760,63 @@ default-value-8
                 (insert-step-instrumenting (callable-tn)
                   ;; Conditionally insert a conditional trap:
                   (when step-instrumenting
-                    (load-symbol-value stepping sb!impl::*stepping*)
+                    (load-symbol-value stepping sb-impl::*stepping*)
                     ;; If it's not 0, trap.
                     (inst beq stepping STEP-DONE-LABEL)
                     (inst nop)
                     ;; CONTEXT-PC will be pointing here when the
                     ;; interrupt is handled, not after the BREAK.
-                    (note-this-location vop :step-before-vop)
-                    ;; Construct a trap code with the low bits from
-                    ;; SINGLE-STEP-AROUND-TRAP and the high bits from
-                    ;; the register number of CALLABLE-TN.
-                    (inst break 0 (logior single-step-around-trap
-                                          (ash (reg-tn-encoding callable-tn)
-                                               5)))
+                    (note-this-location vop :internal-error)
+                    (inst break (reg-tn-encoding callable-tn)
+                          single-step-around-trap)
                     (emit-label step-done-label))))
-
-           ,@(if named
-                 `((sc-case name
-                     (descriptor-reg (move name-pass name))
-                     (control-stack
-                      (inst lw name-pass cfp-tn
-                            (ash (tn-offset name) word-shift))
-                      (do-next-filler))
-                     (constant
-                      (inst lw name-pass code-tn
-                            (- (ash (tn-offset name) word-shift)
-                               other-pointer-lowtag))
-                      (do-next-filler)))
-                   ;; The step instrumenting must be done after
-                   ;; FUNCTION is loaded, but before ENTRY-POINT is
-                   ;; calculated.
-                   (insert-step-instrumenting name-pass)
-                   (inst lw entry-point name-pass
-                         (- (ash fdefn-raw-addr-slot word-shift)
-                            other-pointer-lowtag))
-                   (do-next-filler))
-                 `((sc-case arg-fun
-                     (descriptor-reg (move lexenv arg-fun))
-                     (control-stack
-                      (inst lw lexenv cfp-tn
-                            (ash (tn-offset arg-fun) word-shift))
-                      (do-next-filler))
-                     (constant
-                      (inst lw lexenv code-tn
-                            (- (ash (tn-offset arg-fun) word-shift)
-                               other-pointer-lowtag))
-                      (do-next-filler)))
-                   (inst lw function lexenv
-                         (- (ash closure-fun-slot word-shift)
-                            fun-pointer-lowtag))
-                   (do-next-filler)
-                   ;; The step instrumenting must be done before
-                   ;; after FUNCTION is loaded, but before ENTRY-POINT
-                   ;; is calculated.
-                   (insert-step-instrumenting function)
-                   (inst addu entry-point function
-                         (- (ash simple-fun-code-offset word-shift)
-                            fun-pointer-lowtag))))
+           (declare (ignorable #'insert-step-instrumenting))
+           ,@(case named
+               ((t)
+                `((sc-case name
+                    (descriptor-reg (move name-pass name))
+                    (control-stack
+                     (inst lw name-pass cfp-tn
+                           (tn-byte-offset name))
+                     (do-next-filler))
+                    (constant
+                     (inst lw name-pass code-tn
+                           (- (tn-byte-offset name)
+                              other-pointer-lowtag))
+                     (do-next-filler)))
+                  ;; The step instrumenting must be done after
+                  ;; FUNCTION is loaded, but before ENTRY-POINT is
+                  ;; calculated.
+                  (insert-step-instrumenting name-pass)
+                  (inst lw entry-point name-pass
+                        (- (ash fdefn-raw-addr-slot word-shift)
+                           other-pointer-lowtag))
+                  (do-next-filler)))
+               ((nil)
+                `((sc-case arg-fun
+                    (descriptor-reg (move lexenv arg-fun))
+                    (control-stack
+                     (inst lw lexenv cfp-tn
+                           (tn-byte-offset arg-fun))
+                     (do-next-filler))
+                    (constant
+                     (inst lw lexenv code-tn
+                           (- (tn-byte-offset arg-fun)
+                              other-pointer-lowtag))
+                     (do-next-filler)))
+                  (inst lw function lexenv
+                        (- (ash closure-fun-slot word-shift)
+                           fun-pointer-lowtag))
+                  (do-next-filler)
+                  ;; The step instrumenting must be done before
+                  ;; after FUNCTION is loaded, but before ENTRY-POINT
+                  ;; is calculated.
+                  (insert-step-instrumenting function)
+                  (inst addu entry-point function
+                        (- (ash simple-fun-insts-offset word-shift)
+                           fun-pointer-lowtag))))
+               (:direct
+                `((inst lw entry-point null-tn (static-fun-offset fun)))))
            (loop
              (if (cdr filler)
                  (do-next-filler)
@@ -841,10 +846,13 @@ default-value-8
 
 (define-full-call call nil :fixed nil)
 (define-full-call call-named t :fixed nil)
+(define-full-call static-call-named :direct :fixed nil)
 (define-full-call multiple-call nil :unknown nil)
 (define-full-call multiple-call-named t :unknown nil)
+(define-full-call static-multiple-call-named :direct :unknown nil)
 (define-full-call tail-call nil :tail nil)
 (define-full-call tail-call-named t :tail nil)
+(define-full-call static-tail-call-named :direct :tail nil)
 
 (define-full-call call-variable nil :fixed t)
 (define-full-call multiple-call-variable nil :unknown t)
@@ -864,6 +872,7 @@ default-value-8
   (:temporary (:sc any-reg :offset lexenv-offset :from (:argument 1)) lexenv)
   (:temporary (:sc any-reg :offset ocfp-offset :from (:argument 2)) ocfp)
   (:temporary (:sc any-reg :offset lra-offset :from (:argument 3)) lra)
+  (:temporary (:sc any-reg :offset nl3-offset) temp)
 
   (:vop-var vop)
 
@@ -877,7 +886,8 @@ default-value-8
 
     ;; Clear the number stack if anything is there and jump to the
     ;; assembly-routine that does the bliting.
-    (inst j (make-fixup 'tail-call-variable :assembly-routine))
+    (inst li temp (make-fixup 'tail-call-variable :assembly-routine))
+    (inst j temp)
     (let ((cur-nfp (current-nfp-tn vop)))
       (if cur-nfp
         (inst addu nsp-tn cur-nfp
@@ -981,12 +991,12 @@ default-value-8
   (:temporary (:sc any-reg :offset nl0-offset :from (:argument 2)) vals)
   (:temporary (:sc any-reg :offset nargs-offset :from (:argument 3)) nvals)
   (:temporary (:sc descriptor-reg :offset a0-offset) a0)
+  (:temporary (:sc any-reg :offset nl3-offset) temp)
   (:temporary (:scs (interior-reg)) lip)
 
   (:vop-var vop)
 
   (:generator 13
-    (let ((not-single (gen-label)))
       ;; Clear the number stack.
       (let ((cur-nfp (current-nfp-tn vop)))
         (when cur-nfp
@@ -1004,13 +1014,14 @@ default-value-8
       (lisp-return lra-arg lip :offset 2)
 
       ;; Nope, not the single case.
-      (emit-label not-single)
+    NOT-SINGLE
       (move ocfp ocfp-arg)
       (move lra lra-arg)
       (move vals vals-arg)
 
-      (inst j (make-fixup 'return-multiple :assembly-routine))
-      (move nvals nvals-arg t))))
+      (inst li temp (make-fixup 'return-multiple :assembly-routine))
+      (inst j temp)
+      (emit-nop-or-move nvals nvals-arg)))
 
 ;;;; XEP hackery:
 
@@ -1061,7 +1072,7 @@ default-value-8
       ;; Everything of interest in registers.
       (inst blez count do-regs)
       ;; Initialize dst to be end of stack.
-      (move dst csp-tn t)
+      (emit-nop-or-move dst csp-tn)
       ;; Initialize src to be end of args.
       (inst addu src cfp-tn nargs-tn)
 
@@ -1094,7 +1105,28 @@ default-value-8
 ;;; More args are stored consecutively on the stack, starting
 ;;; immediately at the context pointer.  The context pointer is not
 ;;; typed, so the lowtag is 0.
-(define-full-reffer more-arg * 0 0 (descriptor-reg any-reg) * %more-arg)
+(define-vop (more-arg)
+  (:translate %more-arg)
+  (:policy :fast-safe)
+  (:args (context :scs (descriptor-reg)) (index :scs (any-reg)))
+  (:arg-types * tagged-num)
+  (:temporary (:scs (any-reg)) temp)
+  (:results (value :scs (descriptor-reg any-reg)))
+  (:result-types *)
+  (:generator 5
+   (inst addu temp context index)
+   (loadw value temp)))
+(define-vop (more-arg-c)
+  (:translate %more-arg)
+  (:policy :fast-safe)
+  (:args (context :scs (descriptor-reg)))
+  (:info index)
+  (:arg-types * (:constant (load/store-index 8 0 0)))
+  (:results (value :scs (descriptor-reg any-reg)))
+  (:result-types *)
+  (:generator 4
+    (loadw value context index)))
+
 
 ;;; Turn more arg (context, count) into a list.
 (define-vop (listify-rest-args)
@@ -1119,7 +1151,7 @@ default-value-8
       (move count count-arg)
       ;; Check to see if there are any arguments.
       (inst beq count done)
-      (move result null-tn t)
+      (inst move result null-tn)
 
       ;; We need to do this atomically.
       (pseudo-atomic (pa-flag)
@@ -1167,7 +1199,7 @@ default-value-8
 ;;;
 (define-vop (more-arg-context)
   (:policy :fast-safe)
-  (:translate sb!c::%more-arg-context)
+  (:translate sb-c::%more-arg-context)
   (:args (supplied :scs (any-reg)))
   (:arg-types tagged-num (:constant fixnum))
   (:info fixed)
@@ -1179,64 +1211,39 @@ default-value-8
     (inst addu count supplied (fixnumize (- fixed)))
     (inst subu context csp-tn count)))
 
-
-;;; Signal wrong argument count error if Nargs isn't = to Count.
-;;;
-#!-precise-arg-count-error
-(define-vop (verify-arg-count)
-  (:policy :fast-safe)
-  (:translate sb!c::%verify-arg-count)
-  (:args (nargs :scs (any-reg)))
-  (:arg-types positive-fixnum (:constant t))
-  (:temporary (:scs (any-reg) :type fixnum) temp)
-  (:info count)
-  (:vop-var vop)
-  (:save-p :compute-only)
-  (:generator 3
-    (let ((err-lab
-           (generate-error-code vop 'invalid-arg-count-error nargs)))
-      (cond ((zerop count)
-             (inst bne nargs err-lab)
-             (inst nop))
-            (t
-             (inst li temp (fixnumize count))
-             (inst bne nargs temp err-lab)
-             (inst nop))))))
-
-#!+precise-arg-count-error
 (define-vop (verify-arg-count)
   (:policy :fast-safe)
   (:args (nargs :scs (any-reg)))
-  (:temporary (:scs (any-reg) :type fixnum) temp)
+  (:temporary (:scs (unsigned-reg)) temp)
   (:arg-types positive-fixnum (:constant t) (:constant t))
   (:info min max)
   (:vop-var vop)
   (:save-p :compute-only)
   (:generator 3
-              (let ((err-lab
-                      (generate-error-code vop 'invalid-arg-count-error nargs)))
-                (cond ((not min)
-                       (cond ((zerop max)
-                              (inst bne nargs err-lab))
-                             (t
-                              (inst li temp (fixnumize max))
-                              (inst bne nargs temp err-lab)))
-                       (inst nop))
-                      (max
-                       (when (plusp min)
-                         (inst li temp (fixnumize min))
-                         (inst sltu temp nargs temp)
-                         (inst bne temp err-lab)
-                         (inst nop))
-                       (inst li temp (fixnumize max))
-                       (inst sltu temp temp nargs)
-                       (inst bne temp err-lab)
-                       (inst nop))
-                      ((plusp min)
-                       (inst li temp (fixnumize min))
-                       (inst sltu temp nargs temp)
-                       (inst bne temp err-lab)
-                       (inst nop))))))
+    (let ((err-lab
+            (generate-error-code vop 'invalid-arg-count-error nargs)))
+      (cond ((not min)
+             (cond ((zerop max)
+                    (inst bne nargs err-lab))
+                   (t
+                    (inst li temp (fixnumize max))
+                    (inst bne nargs temp err-lab)))
+             (inst nop))
+            (max
+             (when (plusp min)
+               (inst li temp (fixnumize min))
+               (inst sltu temp nargs temp)
+               (inst bne temp err-lab)
+               (inst nop))
+             (inst li temp (fixnumize max))
+             (inst sltu temp temp nargs)
+             (inst bne temp err-lab)
+             (inst nop))
+            ((plusp min)
+             (inst li temp (fixnumize min))
+             (inst sltu temp nargs temp)
+             (inst bne temp err-lab)
+             (inst nop))))))
 
 ;;; Single-stepping
 
@@ -1245,13 +1252,13 @@ default-value-8
   (:policy :fast-safe)
   (:vop-var vop)
   (:generator 3
-    (load-symbol-value stepping sb!impl::*stepping*)
+    (load-symbol-value stepping sb-impl::*stepping*)
     ;; If it's not 0, trap.
     (inst beq stepping DONE)
     (inst nop)
     ;; CONTEXT-PC will be pointing here when the interrupt is handled,
     ;; not after the BREAK.
-    (note-this-location vop :step-before-vop)
+    (note-this-location vop :internal-error)
     ;; CALLEE-REGISTER-OFFSET isn't needed for before-traps, so we
     ;; can just use a bare SINGLE-STEP-BEFORE-TRAP as the code.
     (inst break 0 single-step-before-trap)
