@@ -35,67 +35,51 @@
 (defmacro load-symbol (reg symbol)
   `(inst addi ,reg null-tn (static-symbol-offset ,symbol)))
 
-(macrolet
-    ((frob (slot)
-       (let ((loader (intern (concatenate 'simple-string
-                                          "LOAD-SYMBOL-"
-                                          (string slot))))
-             (storer (intern (concatenate 'simple-string
-                                          "STORE-SYMBOL-"
-                                          (string slot))))
-             (offset (intern (concatenate 'simple-string
-                                          "SYMBOL-"
-                                          (string slot)
-                                          "-SLOT")
-                             (find-package "SB-VM"))))
-         `(progn
-            (defmacro ,loader (reg symbol)
-              `(progn
-                 ;; Work around the usual lowtag subtraction problem.
-                 (inst li temp-reg-tn
-                       (+ (static-symbol-offset ',symbol)
-                          (ash ,',offset word-shift)
-                          (- other-pointer-lowtag)))
-                 (inst ldx ,reg null-tn temp-reg-tn)))
-            (defmacro ,storer (reg symbol)
-              `(progn
-                 (inst li temp-reg-tn
-                       (+ (static-symbol-offset ',symbol)
-                          (ash ,',offset word-shift)
-                          (- other-pointer-lowtag)))
-                 (inst stdx ,reg null-tn temp-reg-tn)))))))
-  (frob value)
-  (frob function)
+#+sb-thread
+(progn
+  (defun load-tls-index (reg symbol)
+    (inst lwz reg symbol (- #+little-endian 4 other-pointer-lowtag)))
+  (defun store-tls-index (reg symbol)
+    (inst stw reg symbol (- #+little-endian 4 other-pointer-lowtag))))
 
-  ;; FIXME: These are only good for static-symbols, so why not
-  ;; statically-allocate the static-symbol TLS slot indices at
-  ;; cross-compile time so we can just use a fixed offset within the
-  ;; TLS block instead of mucking about with the extra memory access
-  ;; (and temp register, for stores)?
-  #+sb-thread
-  (defmacro load-tl-symbol-value (reg symbol)
-    `(progn
-       (inst lwz ,reg null-tn
-             (+ (static-symbol-offset ',symbol)
-                (ash symbol-tls-index-slot word-shift)
-                (- other-pointer-lowtag)))
-       (inst lwzx ,reg thread-base-tn ,reg)))
-  #-sb-thread
-  (defmacro load-tl-symbol-value (reg symbol)
-    `(load-symbol-value ,reg ,symbol))
+(defmacro load-symbol-value (reg symbol)
+  ;; Work around the usual lowtag subtraction problem.
+  `(progn
+     (inst li temp-reg-tn (+ (static-symbol-offset ',symbol)
+                             (ash symbol-value-slot word-shift)
+                             (- other-pointer-lowtag)))
+     (inst ldx ,reg null-tn temp-reg-tn)))
+(defmacro store-symbol-value (reg symbol)
+  `(progn
+     (inst li temp-reg-tn (+ (static-symbol-offset ',symbol)
+                             (ash symbol-value-slot word-shift)
+                             (- other-pointer-lowtag)))
+     (inst stdx ,reg null-tn temp-reg-tn)))
 
-  #+sb-thread
-  (defmacro store-tl-symbol-value (reg symbol temp)
-    `(progn
-       (inst lwz ,temp null-tn
-             (+ (static-symbol-offset ',symbol)
-                (ash symbol-tls-index-slot word-shift)
-                (- other-pointer-lowtag)))
-       (inst stwx ,reg thread-base-tn ,temp)))
-  #-sb-thread
-  (defmacro store-tl-symbol-value (reg symbol temp)
-    (declare (ignore temp))
-    `(store-symbol-value ,reg ,symbol)))
+;; FIXME: These are only good for static-symbols, so why not
+;; statically-allocate the static-symbol TLS slot indices at
+;; cross-compile time so we can just use a fixed offset within the
+;; TLS block instead of mucking about with the extra memory access
+;; (and temp register, for stores)?
+#+sb-thread
+(progn
+(defmacro load-tl-symbol-value (reg symbol)
+  `(progn
+     (inst lwz ,reg null-tn (+ (static-symbol-offset ',symbol)
+                               (- #+little-endian 4 other-pointer-lowtag)))
+     (inst ldx ,reg thread-base-tn ,reg)))
+(defmacro store-tl-symbol-value (reg symbol temp)
+  `(progn
+     (inst lwz ,temp null-tn (+ (static-symbol-offset ',symbol)
+                                (- #+little-endian 4 other-pointer-lowtag)))
+     (inst stdx ,reg thread-base-tn ,temp))))
+#-sb-thread
+(progn
+(defmacro load-tl-symbol-value (reg symbol)
+  `(load-symbol-value ,reg ,symbol))
+(defmacro store-tl-symbol-value (reg symbol temp)
+  (declare (ignore temp))
+  `(store-symbol-value ,reg ,symbol)))
 
 (defmacro load-type (target source &optional (offset 0))
   "Loads the type bits of a pointer into target independent of
@@ -209,7 +193,7 @@
   #+nil
   (let ((lip (make-random-tn :kind :normal :sc (sc-or-lose 'unsigned-reg)
                              :offset lip-offset)))
-    (inst lr lip (make-fixup 'alloc-tramp :assembly-routine))
+    (inst addi lip null-tn (make-fixup 'alloc-tramp :asm-routine-nil-offset))
     (if (numberp size)
         (inst lr temp-tn size)
         (move temp-tn size))
@@ -231,21 +215,28 @@
     (inst ori result-tn result-tn lowtag)
     (return-from allocation))
 
-  (let ((imm-size (typep size '(unsigned-byte 15)))
-        (region (+ #+sb-thread (* thread-alloc-region-slot n-word-bytes))))
+  (binding* ((imm-size (typep size '(unsigned-byte 15)))
+             ((region-base-tn field-offset)
+               #-sb-thread (values thread-base-tn ; will be STATIC-SPACE-START
+                                   ;; skip over the array header
+                                   (* 2 n-word-bytes))
+               #+sb-thread (values thread-base-tn
+                                   (* thread-alloc-region-slot n-word-bytes))))
+
+    ;; use a spare register because of the usual problem that lw & sw only allow
+    ;; displacements that are a multiple of 4. Otherwise NULL-TN would do.
+    ;; STATIC-SPACE-START can be put into the register using exactly 1 instruction
+    ;; without referencing a code constant, whereas computing the actual base
+    ;; address of the struct would use an LIS + ORI.
+    #-sb-thread (inst lr thread-base-tn static-space-start)
 
     (unless imm-size ; Make temp-tn be the size
       (if (numberp size)
           (inst lr temp-tn size)
           (move temp-tn size)))
 
-    ;; thread-base-tn will point directly to the C variable if no thread
-    #-sb-thread
-    (progn (inst lr thread-base-tn (make-fixup "gc_alloc_region" :foreign-dataref))
-           (inst ld thread-base-tn thread-base-tn 0)) ; because sb-dynamic-core
-
-    (inst ld result-tn thread-base-tn region)
-    (inst ld flag-tn thread-base-tn (+ region n-word-bytes)) ; region->end_addr
+    (inst ld result-tn region-base-tn field-offset)
+    (inst ld flag-tn region-base-tn (+ field-offset n-word-bytes)) ; region->end_addr
 
     (without-scheduling ()
          ;; CAUTION: The C code depends on the exact order of
@@ -269,7 +260,7 @@
 
          ;; The C code depends on this instruction sequence taking up
          ;; one machine instruction.
-         (inst std result-tn thread-base-tn region))
+         (inst std result-tn region-base-tn field-offset))
 
        ;; Should the allocation trap above have fired, the runtime
        ;; arranges for execution to resume here, just after where we
@@ -315,7 +306,8 @@
     (inst andi. temp csp-tn lowtag-mask)
     (inst beq aligned)
     (inst addi csp-tn csp-tn n-word-bytes)
-    (storew zero-tn csp-tn -1)
+    (inst li temp 0)
+    (storew temp csp-tn -1)
     (emit-label aligned)))
 
 
@@ -376,4 +368,5 @@
 
 #+sb-safepoint
 (defun emit-safepoint ()
-  (inst lwz zero-tn null-tn (- (+ gc-safepoint-trap-offset n-word-bytes other-pointer-lowtag))))
+  (inst lwz temp-reg-tn null-tn
+        (- (+ gc-safepoint-trap-offset n-word-bytes other-pointer-lowtag))))
