@@ -56,9 +56,11 @@
   (:result-types fixnum)
   (:temporary (:scs (non-descriptor-reg)) temp)
   (:generator 1
-    (inst lr temp (- (ash (+ instance-slots-offset
-                             (get-dsd-index layout sb-kernel::%bits))
-                          word-shift) instance-pointer-lowtag))
+    (inst lr temp (- (+ #+little-endian 4
+                        (ash (+ instance-slots-offset
+                                (get-dsd-index layout sb-kernel::%bits))
+                             word-shift))
+                     instance-pointer-lowtag))
     (inst lwax res object temp)))
 
 (define-vop (%other-pointer-widetag)
@@ -120,8 +122,7 @@
   (:results (res :scs (any-reg descriptor-reg)))
   (:policy :fast-safe)
   (:generator 1
-    (inst srdi res ptr 1) ; clear the fixnum sign bit
-    (inst rldicr res res 0 (- 63 n-fixnum-tag-bits))))
+    (inst clrrdi res ptr n-fixnum-tag-bits)))
 
 
 ;;;; Allocation
@@ -202,8 +203,8 @@
   (:arg-types signed-num)
   (:policy :fast-safe)
   (:generator 2
-    (inst slwi n n word-shift)
-    (inst lwzx sap thread-base-tn n)))
+    (inst sldi n n word-shift)
+    (inst ldx sap thread-base-tn n)))
 
 (define-vop (halt)
   (:generator 1
