@@ -36,6 +36,27 @@
   (tagify alloc-tn rsp-tn lowtag)
   (values))
 
+;;; For assemfile
+#+(and avx2 sb-xc-host)
+(defvar *avx-registers-used-p* nil)
+
+#+avx2
+(defun avx-registers-used-p ()
+  (or #+sb-xc-host *avx-registers-used-p*
+      (when (and #+sb-xc-host (boundp '*component-being-compiled*))
+        (let ((comp (component-info *component-being-compiled*)))
+          (or (sb-c::ir2-component-avx2-used-p comp)
+              (flet ((used-p (tn)
+                       (do ((tn tn (sb-c::tn-next tn)))
+                           ((null tn))
+                         (when (sc-is tn avx2-reg
+                                      int-avx2-reg
+                                      double-avx2-reg single-avx2-reg)
+                           (return-from avx-registers-used-p
+                             (setf (sb-c::ir2-component-avx2-used-p comp) t))))))
+                (used-p (sb-c::ir2-component-normal-tns comp))
+                (used-p (sb-c::ir2-component-wired-tns comp))))))))
+
 (defun %alloc-tramp (node result-tn size lowtag)
   (cond ((typep size '(and integer (not (signed-byte 32))))
          ;; MOV accepts large immediate operands, PUSH does not
@@ -48,7 +69,12 @@
   ;; But the assembly routine is hand-written, not generated, and it has to match,
   ;; so there's not much that can be done to generalize it.
   (let ((to-r11 (location= result-tn r11-tn)))
-    (invoke-asm-routine 'call (if to-r11 'alloc-tramp-r11 'alloc-tramp) node)
+    (invoke-asm-routine 'call (cond
+                                #+avx2
+                                ((avx-registers-used-p)
+                                 (if to-r11 'alloc-tramp-r11-avx2 'alloc-tramp-avx2))
+                                (t
+                                 (if to-r11 'alloc-tramp-r11 'alloc-tramp))) node)
     (unless to-r11
       (inst pop result-tn)))
   (unless (eql lowtag 0)
@@ -98,13 +124,6 @@
   (aver (and (not (location= alloc-tn temp-reg-tn))
              (or (integerp size) (not (location= size temp-reg-tn)))))
 
-  #+(and (not sb-thread) sb-dynamic-core)
-  ;; We'd need a spare reg in which to load boxed_region from the linkage table.
-  ;; Could push/pop any random register on the stack and own it temporarily,
-  ;; but seeing as nobody cared about this, just punt.
-  (%alloc-tramp node alloc-tn size lowtag)
-
-  #-(and (not sb-thread) sb-dynamic-core)
   ;; Otherwise do the normal inline allocation thing
   (let ((NOT-INLINE (gen-label))
         (DONE (gen-label))
@@ -113,11 +132,13 @@
         ;; thread->alloc_region.free_pointer
         (free-pointer
          #+sb-thread (thread-slot-ea thread-alloc-region-slot)
-         #-sb-thread (ea (make-fixup "gc_alloc_region" :foreign)))
+         #-sb-thread (ea (+ static-space-start
+                            (ash vector-data-offset word-shift))))
         ;; thread->alloc_region.end_addr
         (end-addr
          #+sb-thread (thread-slot-ea (1+ thread-alloc-region-slot))
-         #-sb-thread (ea (make-fixup "gc_alloc_region" :foreign 8))))
+         #-sb-thread (ea (+ static-space-start
+                            (ash (1+ vector-data-offset) word-shift)))))
 
     (cond ((or in-elsewhere
                ;; large objects will never be made in a per-thread region
@@ -532,8 +553,9 @@
                        temp)
                 result 0 fun-pointer-lowtag (not stack-allocate-p)))
      ;; Done with pseudo-atomic
-     (inst lea temp (rip-relative-ea label (ash simple-fun-insts-offset word-shift)))
-     (storew temp result closure-fun-slot fun-pointer-lowtag))))
+     (when label
+       (inst lea temp (rip-relative-ea label (ash simple-fun-insts-offset word-shift)))
+       (storew temp result closure-fun-slot fun-pointer-lowtag)))))
 
 ;;; The compiler likes to be able to directly make value cells.
 (define-vop (make-value-cell)
@@ -604,14 +626,14 @@
 ;;; Exactly 4 allocators are rendered via this vop:
 ;;;  BIGNUM               (%ALLOCATE-BIGNUM)
 ;;;  FUNCALLABLE-INSTANCE (%MAKE-FUNCALLABLE-INSTANCE)
-;;;  CLOSURE              (%COPY-CLOSURE)
+;;;  CLOSURE              (%ALLOC-CLOSURE)
 ;;;  INSTANCE             (%MAKE-INSTANCE)
 ;;; WORDS accounts for the mandatory slots *including* the header.
 ;;; EXTRA is the variable payload, also measured in words.
 (define-vop (var-alloc)
   (:args (extra :scs (any-reg)))
   (:arg-types positive-fixnum)
-  (:info name words type lowtag)
+  (:info name words type lowtag stack-allocate-p)
   (:ignore name)
   (:results (result :scs (descriptor-reg) :from (:eval 1)))
   (:temporary (:sc unsigned-reg :from :eval :to (:eval 1)) bytes)
@@ -629,10 +651,14 @@
       (inst lea operand-size header                    ; (w-1 << 8) | type
             (ea (+ (ash -2 n-widetag-bits) type) header))
       (inst and operand-size bytes (lognot lowtag-mask)))
-      (instrument-alloc bytes node)
-      (pseudo-atomic ()
-       (allocation result bytes node nil lowtag)
-       (storew header result 0 lowtag))))
+      (cond (stack-allocate-p
+             (stack-allocation result bytes lowtag)
+             (storew header result 0 lowtag))
+            (t
+             (instrument-alloc bytes node)
+             (pseudo-atomic ()
+              (allocation result bytes node nil lowtag)
+              (storew header result 0 lowtag))))))
 
 (macrolet ((c-call (name)
              `(let ((c-fun (make-fixup ,name :foreign)))
