@@ -9,16 +9,167 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
-;;;; CONS, LIST and LIST*
-(defoptimizer (cons stack-allocate-result) ((&rest args))
-  t)
-(defoptimizer (list stack-allocate-result) ((&rest args))
-  (not (null args)))
-(defoptimizer (list* stack-allocate-result) ((&rest args))
-  (not (null (rest args))))
+;;;; allocation helpers
 
+;;; Allocation within alloc_region (which is thread local) can be done
+;;; inline.  If the alloc_region is overflown allocation is done by
+;;; calling the C alloc() function.
+
+;;; C calls for allocation don't /seem/ to make an awful lot of
+;;; difference to speed. On pure consing it's about a 25%
+;;; gain. Guessing from historical context, it looks like inline
+;;; allocation was introduced before pseudo-atomic, at which time all
+;;; calls to alloc() would have needed a syscall to mask signals for
+;;; the duration.  Now we have pseudoatomic there's no need for that
+;;; overhead.
+
+(defun stack-allocation (alloc-tn size lowtag)
+  (aver (not (location= alloc-tn esp-tn)))
+  (inst sub esp-tn size)
+  ;; FIXME: SIZE _should_ be double-word aligned (suggested but
+  ;; unfortunately not enforced by PAD-DATA-BLOCK and
+  ;; FIXED-ALLOC), so that ESP is always divisible by 8 (for
+  ;; 32-bit lispobjs).  In that case, this AND instruction is
+  ;; unnecessary and could be removed.  If not, explain why.  -- CSR,
+  ;; 2004-03-30
+  (inst and esp-tn (lognot lowtag-mask))
+  (inst lea alloc-tn (make-ea :byte :base esp-tn :disp lowtag))
+  (values))
+
+(defun allocation-notinline (alloc-tn size)
+  (let* ((alloc-tn-offset (tn-offset alloc-tn))
+         ;; C call to allocate via dispatch routines. Each
+         ;; destination has a special entry point. The size may be a
+         ;; register or a constant.
+         (tn-text (ecase alloc-tn-offset
+                    (#.eax-offset "eax")
+                    (#.ecx-offset "ecx")
+                    (#.edx-offset "edx")
+                    (#.ebx-offset "ebx")
+                    (#.esi-offset "esi")
+                    (#.edi-offset "edi")))
+         (size-text (case size (8 "8_") (16 "16_") (t ""))))
+    (unless (or (eql size 8) (eql size 16))
+      (unless (and (tn-p size) (location= alloc-tn size))
+        (inst mov alloc-tn size)))
+    (inst call (make-fixup (concatenate 'string
+                                         "alloc_" size-text
+                                         "to_" tn-text)
+                           :foreign))))
+
+(defun allocation-inline (alloc-tn size)
+  (let* ((ok (gen-label))
+         (done (gen-label))
+         #+(and sb-thread win32)
+         (scratch-tns (loop for my-tn in `(,eax-tn ,ebx-tn ,edx-tn ,ecx-tn)
+                            when (and (not (location= alloc-tn my-tn))
+                                      (or (not (tn-p size))
+                                          (not (location= size my-tn))))
+                            collect my-tn))
+         (tls-prefix #+sb-thread :fs)
+         #+(and sb-thread win32) (scratch-tn (pop scratch-tns))
+         #+(and sb-thread win32) (swap-tn (pop scratch-tns))
+         (free-pointer
+           ;; thread->alloc_region.free_pointer
+           (make-ea :dword
+                    :base (or #+(and sb-thread win32)
+                              scratch-tn)
+                    :disp
+                    #+sb-thread (* n-word-bytes thread-alloc-region-slot)
+                    #-sb-thread (+ static-space-start
+                                   (ash vector-data-offset word-shift))))
+         (end-addr
+            ;; thread->alloc_region.end_addr
+           (make-ea :dword
+                    :base (or #+(and sb-thread win32)
+                              scratch-tn)
+                    :disp
+                    #+sb-thread (* n-word-bytes (1+ thread-alloc-region-slot))
+                    #-sb-thread (+ static-space-start
+                                   (ash (1+ vector-data-offset) word-shift)))))
+    (unless (and (tn-p size) (location= alloc-tn size))
+      (inst mov alloc-tn size))
+    #+(and sb-thread win32)
+    (progn
+      (inst push scratch-tn)
+      (inst push swap-tn)
+      (inst mov scratch-tn
+            (make-ea :dword :disp
+                     +win32-tib-arbitrary-field-offset+) tls-prefix)
+      (setf tls-prefix nil))
+    (inst add alloc-tn free-pointer tls-prefix)
+    (inst cmp alloc-tn end-addr tls-prefix)
+    (inst jmp :be ok)
+    (let ((dst (ecase (tn-offset alloc-tn)
+                 (#.eax-offset "alloc_overflow_eax")
+                 (#.ecx-offset "alloc_overflow_ecx")
+                 (#.edx-offset "alloc_overflow_edx")
+                 (#.ebx-offset "alloc_overflow_ebx")
+                 (#.esi-offset "alloc_overflow_esi")
+                 (#.edi-offset "alloc_overflow_edi"))))
+      (inst call (make-fixup dst :foreign)))
+    (inst jmp done)
+    (emit-label ok)
+    ;; Swap ALLOC-TN and FREE-POINTER
+    (cond ((and (tn-p size) (location= alloc-tn size))
+           ;; XCHG is extremely slow, use the xor swap trick
+           #-(and sb-thread win32)
+           (progn
+             (inst xor alloc-tn free-pointer tls-prefix)
+             (inst xor free-pointer alloc-tn tls-prefix)
+             (inst xor alloc-tn free-pointer tls-prefix))
+           #+(and sb-thread win32)
+           (progn
+             (inst mov swap-tn free-pointer tls-prefix)
+             (inst mov free-pointer alloc-tn tls-prefix)
+             (inst mov alloc-tn swap-tn)))
+          (t
+           ;; It's easier if SIZE is still available.
+           (inst mov free-pointer alloc-tn tls-prefix)
+           (inst sub alloc-tn size)))
+    (emit-label done)
+    #+(and sb-thread win32)
+    (progn
+      (inst pop swap-tn)
+      (inst pop scratch-tn))
+    (values)))
+
+;;; Emit code to allocate an object with a size in bytes given by
+;;; SIZE.  The size may be an integer or a TN. If Inline is a VOP
+;;; node-var then it is used to make an appropriate speed vs size
+;;; decision.
+
+;;; Allocation should only be used inside a pseudo-atomic section, which
+;;; should also cover subsequent initialization of the object.
+
+;;; (FIXME: so why aren't we asserting this?)
+
+(defun allocation (alloc-tn size node &optional dynamic-extent lowtag)
+  (declare (ignorable node))
+  (cond
+    (dynamic-extent
+     (stack-allocation alloc-tn size lowtag))
+    ((or (null node) (policy node (>= speed space)))
+     (allocation-inline alloc-tn size))
+    (t
+     (allocation-notinline alloc-tn size)))
+  (when (and lowtag (not dynamic-extent))
+    (inst lea alloc-tn (make-ea :byte :base alloc-tn :disp lowtag)))
+  (values))
+
+;;; Allocate an other-pointer object of fixed SIZE with a single word
+;;; header having the specified WIDETAG value. The result is placed in
+;;; RESULT-TN.
+(defun alloc-other (result-tn widetag size node &optional stack-allocate-p)
+  (pseudo-atomic (:elide-if stack-allocate-p)
+      (allocation result-tn (pad-data-block size) node stack-allocate-p
+                  other-pointer-lowtag)
+      (storew (compute-object-header size widetag)
+              result-tn 0 other-pointer-lowtag)))
+
+;;;; CONS, LIST and LIST*
 (define-vop (list-or-list*)
   (:args (things :more t))
   (:temporary (:sc unsigned-reg) ptr temp)
@@ -45,9 +196,8 @@
                              temp))))
                      (storew reg ,list ,slot list-pointer-lowtag))))
              (let ((cons-cells (if star (1- num) num))
-                   (stack-allocate-p (awhen (sb!c::node-lvar node)
-                                       (sb!c::lvar-dynamic-extent it))))
-               (maybe-pseudo-atomic stack-allocate-p
+                   (stack-allocate-p (node-stack-allocate-p node)))
+               (pseudo-atomic (:elide-if stack-allocate-p)
                 (allocation res (* (pad-data-block cons-size) cons-cells) node
                             stack-allocate-p list-pointer-lowtag)
                 (move ptr res)
@@ -85,6 +235,7 @@
               positive-fixnum
               positive-fixnum)
   (:policy :fast-safe)
+  (:node-var node)
   (:generator 100
     (let ((size (sc-case words
                   (immediate
@@ -99,13 +250,13 @@
                                                    n-word-bytes))))
                    (inst and result (lognot lowtag-mask))
                    result))))
-      (pseudo-atomic
-       (allocation result size)
+      (pseudo-atomic ()
+       (allocation result size node)
        (inst lea result (make-ea :byte :base result :disp other-pointer-lowtag))
        (sc-case type
          (immediate
-          (aver (typep (tn-value type) '(unsigned-byte 8)))
-          (storeb (tn-value type) result 0 other-pointer-lowtag))
+          (aver (typep (tn-value type) '(unsigned-byte 9)))
+          (store-widetag (tn-value type) result 0 other-pointer-lowtag))
          (t
           (storew type result 0 other-pointer-lowtag)))
        (sc-case length
@@ -113,7 +264,7 @@
           (let ((fixnum-length (fixnumize (tn-value length))))
             (typecase fixnum-length
               ((unsigned-byte 8)
-               (storeb fixnum-length result
+               (store-widetag fixnum-length result
                        vector-length-slot other-pointer-lowtag))
               (t
                (storew fixnum-length result
@@ -122,19 +273,18 @@
           (storew length result vector-length-slot other-pointer-lowtag)))))))
 
 (define-vop (allocate-vector-on-stack)
-  (:args (type :scs (unsigned-reg immediate))
-         (length :scs (any-reg))
+  (:args (type :scs (unsigned-reg immediate) :to :save)
+         (length :scs (any-reg) :to :eval :target zero)
          (words :scs (any-reg) :target ecx))
   (:temporary (:sc any-reg :offset ecx-offset :from (:argument 2)) ecx)
-  (:temporary (:sc any-reg :offset eax-offset :from (:argument 2)) zero)
-  (:temporary (:sc any-reg :offset edi-offset :from (:argument 0)) res)
+  (:temporary (:sc any-reg :offset eax-offset :from :eval) zero)
+  (:temporary (:sc any-reg :offset edi-offset) res)
   (:results (result :scs (descriptor-reg) :from :load))
   (:arg-types positive-fixnum
               positive-fixnum
               positive-fixnum)
   (:translate allocate-vector)
   (:policy :fast-safe)
-  (:node-var node)
   (:generator 100
     (inst lea result (make-ea :byte :base words :disp
                               (+ (1- (ash 1 n-lowtag-bits))
@@ -143,7 +293,7 @@
     ;; FIXME: It would be good to check for stack overflow here.
     (move ecx words)
     (inst shr ecx n-fixnum-tag-bits)
-    (allocation result result node t other-pointer-lowtag)
+    (stack-allocation result result other-pointer-lowtag)
     (inst cld)
     (inst lea res
           (make-ea :byte :base result :disp (- (* vector-data-offset n-word-bytes)
@@ -151,7 +301,7 @@
     (sc-case type
       (immediate
        (aver (typep (tn-value type) '(unsigned-byte 8)))
-       (storeb (tn-value type) result 0 other-pointer-lowtag))
+       (store-widetag (tn-value type) result 0 other-pointer-lowtag))
       (t
        (storew type result 0 other-pointer-lowtag)))
     (storew length result vector-length-slot other-pointer-lowtag)
@@ -159,68 +309,6 @@
     (inst rep)
     (inst stos zero)))
 
-(in-package "SB!C")
-
-(defoptimizer (allocate-vector stack-allocate-result)
-    ((type length words) node)
-  (ecase (policy node stack-allocate-vector)
-    (0 nil)
-    ((1 2)
-     ;; a vector object should fit in one page
-     (values-subtypep (lvar-derived-type words)
-                      (load-time-value
-                       (specifier-type `(integer 0 ,(- (/ sb!vm::*backend-page-size*
-                                                          sb!vm:n-word-bytes)
-                                                       sb!vm:vector-data-offset))))))
-    (3 t)))
-
-(defoptimizer (allocate-vector ltn-annotate) ((type length words) call ltn-policy)
-  (let ((args (basic-combination-args call))
-        (template (template-or-lose (if (awhen (node-lvar call)
-                                          (lvar-dynamic-extent it))
-                                        'sb!vm::allocate-vector-on-stack
-                                        'sb!vm::allocate-vector-on-heap))))
-    (dolist (arg args)
-      (setf (lvar-info arg)
-            (make-ir2-lvar (primitive-type (lvar-type arg)))))
-    (unless (is-ok-template-use template call (ltn-policy-safe-p ltn-policy))
-      (ltn-default-call call)
-      (return-from allocate-vector-ltn-annotate-optimizer (values)))
-    (setf (basic-combination-info call) template)
-    (setf (node-tail-p call) nil)
-
-    (dolist (arg args)
-      (annotate-1-value-lvar arg))))
-
-(in-package "SB!VM")
-
-;;;
-(define-vop (allocate-code-object)
-  (:args (boxed-arg :scs (any-reg) :target boxed)
-         (unboxed-arg :scs (any-reg) :target unboxed))
-  (:results (result :scs (descriptor-reg) :from :eval))
-  (:temporary (:sc unsigned-reg :from (:argument 0)) boxed)
-  (:temporary (:sc unsigned-reg :from (:argument 1)) unboxed)
-  (:node-var node)
-  (:generator 100
-    (move boxed boxed-arg)
-    (inst add boxed (fixnumize (1+ code-trace-table-offset-slot)))
-    (inst and boxed (lognot lowtag-mask))
-    (move unboxed unboxed-arg)
-    (inst shr unboxed word-shift)
-    (inst add unboxed lowtag-mask)
-    (inst and unboxed (lognot lowtag-mask))
-    (inst mov result boxed)
-    (inst add result unboxed)
-    (pseudo-atomic
-     (allocation result result node)
-     (inst lea result (make-ea :byte :base result :disp other-pointer-lowtag))
-     (inst shl boxed (- n-widetag-bits word-shift))
-     (inst or boxed code-header-widetag)
-     (storew boxed result 0 other-pointer-lowtag)
-     (storew unboxed result code-code-size-slot other-pointer-lowtag)
-     (storew nil-value result code-entry-points-slot other-pointer-lowtag))
-    (storew nil-value result code-debug-info-slot other-pointer-lowtag)))
 
 (define-vop (make-fdefn)
   (:policy :fast-safe)
@@ -229,28 +317,31 @@
   (:results (result :scs (descriptor-reg) :from :argument))
   (:node-var node)
   (:generator 37
-    (with-fixed-allocation (result fdefn-widetag fdefn-size node)
-      (storew name result fdefn-name-slot other-pointer-lowtag)
-      (storew nil-value result fdefn-fun-slot other-pointer-lowtag)
-      (storew (make-fixup "undefined_tramp" :foreign)
-              result fdefn-raw-addr-slot other-pointer-lowtag))))
+    (alloc-other result fdefn-widetag fdefn-size node)
+    (storew name result fdefn-name-slot other-pointer-lowtag)
+    (storew nil-value result fdefn-fun-slot other-pointer-lowtag)
+    (storew (make-fixup 'undefined-tramp :assembly-routine)
+            result fdefn-raw-addr-slot other-pointer-lowtag)))
 
 (define-vop (make-closure)
   (:args (function :to :save :scs (descriptor-reg)))
-  (:info length stack-allocate-p)
+  (:info label length stack-allocate-p)
+  (:ignore label)
   (:temporary (:sc any-reg) temp)
   (:results (result :scs (descriptor-reg)))
   (:node-var node)
   (:generator 10
-   (maybe-pseudo-atomic stack-allocate-p
+   (pseudo-atomic (:elide-if stack-allocate-p)
      (let ((size (+ length closure-info-offset)))
        (allocation result (pad-data-block size) node
                    stack-allocate-p
                    fun-pointer-lowtag)
-       (storew (logior (ash (1- size) n-widetag-bits) closure-header-widetag)
-               result 0 fun-pointer-lowtag))
-    (loadw temp function closure-fun-slot fun-pointer-lowtag)
-    (storew temp result closure-fun-slot fun-pointer-lowtag))))
+       (storew (logior (ash (1- size) n-widetag-bits) closure-widetag)
+               result 0 fun-pointer-lowtag)))
+   ;; Done with pseudo-atomic
+   (inst lea temp (object-slot-ea function simple-fun-insts-offset
+                                           fun-pointer-lowtag))
+   (storew temp result closure-fun-slot fun-pointer-lowtag)))
 
 ;;; The compiler likes to be able to directly make value cells.
 (define-vop (make-value-cell)
@@ -259,15 +350,14 @@
   (:info stack-allocate-p)
   (:node-var node)
   (:generator 10
-    (with-fixed-allocation
-        (result value-cell-header-widetag value-cell-size node stack-allocate-p)
-      (storew value result value-cell-value-slot other-pointer-lowtag))))
+    (alloc-other result value-cell-widetag value-cell-size node stack-allocate-p)
+    (storew value result value-cell-value-slot other-pointer-lowtag)))
 
 ;;;; automatic allocators for primitive objects
 
 (define-vop (make-unbound-marker)
   (:args)
-  (:results (result :scs (any-reg)))
+  (:results (result :scs (descriptor-reg any-reg)))
   (:generator 1
     (inst mov result unbound-marker-widetag)))
 
@@ -275,7 +365,7 @@
   (:args)
   (:results (result :scs (any-reg)))
   (:generator 1
-    (inst lea result (make-fixup "funcallable_instance_tramp" :foreign))))
+    (inst mov result (make-fixup 'funcallable-instance-tramp :assembly-routine))))
 
 (define-vop (fixed-alloc)
   (:args)
@@ -298,13 +388,13 @@
                #.(loop for offset in *dword-regs*
                     collect `(,offset
                               ',(intern (format nil "ALLOCATE-CONS-TO-~A"
-                                                (svref *dword-register-names*
+                                                (svref +dword-register-names+
                                                        offset)))) into cases
                     finally (return `(case (tn-offset result)
                                        ,@cases)))))
           (aver (null type))
           (inst call (make-fixup dst :assembly-routine)))
-        (maybe-pseudo-atomic stack-allocate-p
+        (pseudo-atomic (:elide-if stack-allocate-p)
          (allocation result (pad-data-block words) node stack-allocate-p lowtag)
          (when type
            (storew (logior (ash (1- words) n-widetag-bits) type)
@@ -315,7 +405,7 @@
 (define-vop (var-alloc)
   (:args (extra :scs (any-reg)))
   (:arg-types positive-fixnum)
-  (:info name words type lowtag)
+  (:info name words type lowtag stack-allocate-p)
   (:ignore name)
   (:results (result :scs (descriptor-reg) :from (:eval 1)))
   (:temporary (:sc any-reg :from :eval :to (:eval 1)) bytes)
@@ -329,7 +419,6 @@
     (inst lea header                    ; (w-1 << 8) | type
           (make-ea :dword :base header :disp (+ (ash -2 n-widetag-bits) type)))
     (inst and bytes (lognot lowtag-mask))
-    (pseudo-atomic
-     (allocation result bytes node)
-     (inst lea result (make-ea :byte :base result :disp lowtag))
+    (pseudo-atomic (:elide-if stack-allocate-p)
+     (allocation result bytes node stack-allocate-p lowtag)
      (storew header result 0 lowtag))))
