@@ -121,10 +121,7 @@
   (:policy :fast-safe)
   (:generator 1
     (move res ptr)
-    ;; Mask the lowtag, and shift the whole address into a positive
-    ;; fixnum.
-    (inst and res (lognot lowtag-mask))
-    (inst shr res 1)))
+    (inst and res (lognot fixnum-tag-mask))))
 
 ;;;; allocation
 
@@ -235,7 +232,7 @@
     (inst lea eax (make-ea :dword :base res :disp (- list-pointer-lowtag)))
     (emit-optimized-test-inst eax lowtag-mask)
     (inst cmov :e res
-          (make-ea-for-object-slot res cons-cdr-slot list-pointer-lowtag))))
+          (object-slot-ea res cons-cdr-slot list-pointer-lowtag))))
 (define-vop (symbol-plist)
   (:policy :fast-safe)
   (:translate symbol-plist)
