@@ -1,6 +1,6 @@
 ;;;; In support of PCL we compile some things into the cold image.
 ;;;; Not only does this simplify the PCL bootstrap ever so slightly,
-;;;; it is nice to be able to test for types SB!PCL::%METHOD-FUNCTION
+;;;; it is nice to be able to test for types SB-PCL::%METHOD-FUNCTION
 ;;;; and CLASS (neither of which will have any instances too early).
 
 ;;;; This software is part of the SBCL system. See the README file for more
@@ -26,7 +26,7 @@
 ;;;; warranty about the software, its performance or its conformity to any
 ;;;; specification.
 
-(in-package "SB!PCL")
+(in-package "SB-PCL")
 
 
 ;;; method function stuff.
@@ -93,12 +93,34 @@
     (short-method-combination short-method-combination-p)))
 
 #+sb-xc-host
+(progn
+;;; Create #<SB-KERNEL::CONDITION-CLASSOID CONDITION>
+;;; so that we can successfully parse the type specifier
+;;; CONDITION-DESIGNATOR-HEAD which expands to
+;;; (or format-control symbol condition sb-pcl::condition-class).
+;;; Compiling any ERROR or WARN call eagerly looks up and re-parses
+;;; the global ftype, and we don't want to see unknown types.
+(let* ((name 'condition)
+       (classoid (sb-kernel::make-condition-classoid :name name))
+       (cell (sb-kernel::make-classoid-cell name classoid))
+       (layout (make-layout
+                (randomish-layout-clos-hash name)
+                classoid
+                :inherits (vector (find-layout 't))
+                :depthoid 1
+                :length (+ sb-vm:instance-data-start 2)
+                :invalid nil)))
+  (setf (classoid-layout classoid) layout
+        (info :type :classoid-cell name) cell
+        (info :type :kind name) :instance))
+
+;;; Create classoids that correspond with some CLOS classes
 (flet ((create-fake-classoid (name fun-p)
          (let* ((classoid (make-standard-classoid :name name))
-                (cell (sb!kernel::make-classoid-cell name classoid))
+                (cell (sb-kernel::make-classoid-cell name classoid))
                 (layout
                  (make-layout
-                  :classoid classoid
+                  (randomish-layout-clos-hash name) classoid
                   :inherits (map 'vector #'find-layout
                                  (cons t (if fun-p '(function))))
                   :length 0 ; don't care
@@ -118,6 +140,7 @@
       (create-fake-classoid name
                             (memq name '(standard-generic-function
                                          generic-function))))))
+) ; end PROGN
 
 ;;; BIG FAT WARNING: These predicates can't in general be called prior to the
 ;;; definition of the class which they test. However in carefully controlled
