@@ -60,7 +60,7 @@
 os_vm_size_t dynamic_space_size = DEFAULT_DYNAMIC_SPACE_SIZE;
 os_vm_size_t thread_control_stack_size = DEFAULT_CONTROL_STACK_SIZE;
 
-sword_t (*scavtab[256])(lispobj *where, lispobj object);
+sword_t (*const scavtab[256])(lispobj *where, lispobj object);
 
 // "Transport" functions are responsible for deciding where to copy an object
 // and how many bytes to copy (usually the sizing function is inlined into the
@@ -133,7 +133,7 @@ static inline void scav1(lispobj* object_ptr, lispobj object)
         if (page_table[page].gen == from_space && !pinned_p(object, page))
             FIX_POINTER();
     } else if (immobile_space_p(object)) {
-        lispobj *ptr = native_pointer(object);
+        lispobj *ptr = base_pointer(object);
         if (immobile_obj_gen_bits(ptr) == from_space)
             enliven_immobile_obj(ptr, 1);
     }
@@ -306,6 +306,18 @@ trans_code(struct code *code)
     struct code *new_code = (struct code *) native_pointer(l_new_code);
     uword_t displacement = l_new_code - l_code;
 
+#if defined LISP_FEATURE_PPC || defined LISP_FEATURE_PPC64 || \
+    defined LISP_FEATURE_X86 || defined LISP_FEATURE_X86_64
+    // Fixup absolute jump tables. These aren't recorded in code->fixups
+    // because we don't need to denote an arbitrary set of places in the code.
+    // The count alone suffices. A GC immediately after creating the code
+    // could cause us to observe some 0 words here. Those should be ignored.
+    lispobj* jump_table = code_jumptable_start(new_code);
+    int count = jumptable_count(jump_table);
+    int i;
+    for (i = 1; i < count; ++i)
+        if (jump_table[i]) jump_table[i] += displacement;
+#endif
     for_each_simple_fun(i, new_fun, new_code, 1, {
         // Calculate the old raw function pointer
         struct simple_fun* old_fun = (struct simple_fun*)((char*)new_fun - displacement);
@@ -315,14 +327,12 @@ trans_code(struct code *code)
                                    make_lispobj(new_fun, FUN_POINTER_LOWTAG));
         }
     })
+    gencgc_apply_code_fixups(code, new_code);
 #ifdef LISP_FEATURE_GENCGC
     /* Cheneygc doesn't need this os_flush_icache, it flushes the whole
        spaces once when all copying is done. */
     os_flush_icache(code_text_start(new_code), code_text_size(new_code));
 #endif
-
-    gencgc_apply_code_fixups(code, new_code);
-
     return new_code;
 }
 
@@ -637,7 +647,7 @@ scav_instance(lispobj *where, lispobj header)
     if (instance_layout(where)) {
         lispobj *layout = native_pointer(instance_layout(where));
 #ifdef LISP_FEATURE_COMPACT_INSTANCE_HEADER
-        if (__immobile_obj_gen_bits(layout) == from_space)
+        if (immobile_obj_gen_bits(layout) == from_space)
             enliven_immobile_obj(layout, 1);
 #else
         if (forwarding_pointer_p(layout))
@@ -903,7 +913,7 @@ trans_weak_pointer(lispobj object)
         cell = forwarding_pointer_p(native) ? \
                LOW_WORD(forwarding_pointer_value(native)) : broken; \
     else if (immobile_space_p(pointee)) { \
-        if (immobile_obj_gen_bits(native) == from_space) cell = broken; \
+        if (immobile_obj_gen_bits(base_pointer(pointee)) == from_space) cell = broken; \
     }
 
 void smash_weak_pointers(void)
@@ -997,7 +1007,7 @@ static inline int pointer_survived_gc_yet(lispobj obj)
                pinned_p(obj, page_index);
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
     if (immobile_space_p(obj))
-        return immobile_obj_gen_bits(native_pointer(obj)) != from_space;
+        return immobile_obj_gen_bits(base_pointer(obj)) != from_space;
 #endif
     return 1;
 #endif
@@ -1477,7 +1487,7 @@ void cull_weak_hash_tables(int (*alivep[5])(lispobj,lispobj))
         NON_FAULTING_STORE(table->next_weak_hash_table = NIL,
                            &table->next_weak_hash_table);
         cull_weak_hash_table(table, alivep[hashtable_weakness(table)],
-                             pair_follow_fps);
+                             compacting_p() ? pair_follow_fps : 0);
     }
     weak_hash_tables = NULL;
     /* Reset weak_objects only if the count is nonzero.
@@ -1531,6 +1541,7 @@ boolean valid_widetag_p(unsigned char widetag) {
  * initialization
  */
 
+sword_t scav_weak_pointer(lispobj *where, lispobj object);
 #include "genesis/gc-tables.h"
 
 /* Find the code object for the given pc, or return NULL on
@@ -1602,14 +1613,6 @@ int simple_fun_index(struct code* code, struct simple_fun *fun)
     return -1;
 }
 
-lispobj simple_fun_name(struct simple_fun* fun)
-{
-    struct code* code = (struct code*)fun_code_header((lispobj*)fun);
-    int index = simple_fun_index(code, fun);
-    if (index < 0) return 0;
-    return code->constants[CODE_SLOTS_PER_SIMPLE_FUN*index];
-}
-
 /* Helper for valid_lisp_pointer_p (below) and
  * conservative_root_p (gencgc).
  *
@@ -1644,7 +1647,10 @@ properly_tagged_p_internal(lispobj pointer, lispobj *start_addr)
     if (lowtag && make_lispobj(start_addr, lowtag) == pointer)
         return 1; // instant win
 
-    if (widetag == CODE_HEADER_WIDETAG) {
+    // debug_info must be assigned prior to examining a code object for simple-funs.
+    // This is how we distinguish objects that are partyway through construction.
+    // It would be wrong to read garbage bytes from the simple-fun table.
+    if (widetag == CODE_HEADER_WIDETAG && ((struct code*)start_addr)->debug_info) {
         if (functionp(pointer)) {
             lispobj* potential_fun = FUNCTION(pointer);
             if (widetag_of(potential_fun) == SIMPLE_FUN_WIDETAG &&
@@ -2063,13 +2069,14 @@ pair_interior_pointer(os_context_t *context, uword_t pointer,
     }
 #if 0
     if (*register_pair >= 0)
-        fprintf(stderr, "p_i_p: %-3s=%#lx based on %s=%lx + %lx\n",
-                regname, pointer,
+        fprintf(stderr, "pair_interior_ptr: %-3s=%p based on %s=%p + %x\n",
+                regname, (void*)pointer,
                 lisp_register_names[*register_pair],
-                *os_context_register_addr(context, *register_pair),
-                *saved_offset);
+                (void*)(((uword_t)*os_context_register_addr(context, *register_pair))
+                        & ~LOWTAG_MASK),
+                (int)*saved_offset);
     else
-        fprintf(stderr, "p_i_p: %-3s=%#lx not based\n", regname, pointer);
+        fprintf(stderr, "pair_interior_ptr: %-3s=%#lx not based\n", regname, pointer);
 #endif
 }
 
