@@ -17,11 +17,14 @@
 
 #include "genesis/weak-pointer.h"
 #include "immobile-space.h"
+#include "code.h"
 
 // Gencgc distinguishes between "quick" and "ordinary" requests.
 // Even on cheneygc we need this flag, but it's actually just ignored.
 #define ALLOC_QUICK 1
 
+#define CUSTOM_GC_SCAVENGE_FLAG 0x800000
+
 #ifdef LISP_FEATURE_GENCGC
 #include "gencgc-alloc-region.h"
 void *
@@ -68,8 +71,8 @@ gc_general_copy_object(lispobj object, long nwords, int page_type_flag)
     return make_lispobj(new, lowtag_of(object));
 }
 
-extern sword_t (*scavtab[256])(lispobj *where, lispobj object);
-extern struct weak_pointer *weak_pointers; /* in gc-common.c */
+extern sword_t (*const scavtab[256])(lispobj *where, lispobj object);
+extern struct cons *weak_vectors; /* in gc-common.c */
 extern struct hash_table *weak_hash_tables; /* in gc-common.c */
 
 // These next two are prototyped for both GCs
@@ -77,6 +80,7 @@ extern struct hash_table *weak_hash_tables; /* in gc-common.c */
 void gc_mark_range(lispobj*start, long count);
 void gc_mark_obj(lispobj);
 void gc_dispose_private_pages();
+void add_to_weak_vector_list(lispobj* vector, lispobj header);
 
 extern void heap_scavenge(lispobj *start, lispobj *limit);
 extern sword_t scavenge(lispobj *start, sword_t n_words);
@@ -84,10 +88,10 @@ extern void scavenge_interrupt_contexts(struct thread *thread);
 extern void scav_binding_stack(lispobj*, lispobj*, void(*)(lispobj));
 extern void scan_binding_stack(void);
 extern void cull_weak_hash_tables(int (*[4])(lispobj,lispobj));
-extern void scan_weak_pointers(void);
-extern boolean scav_hash_table_entries(struct hash_table *hash_table,
-                                       int (*)(lispobj,lispobj),
-                                       void (*)(lispobj*));
+extern void smash_weak_pointers(void);
+extern boolean scan_weak_hashtable(struct hash_table *hash_table,
+                                   int (*)(lispobj,lispobj),
+                                   void (*)(lispobj*));
 extern int (*weak_ht_alivep_funs[4])(lispobj,lispobj);
 extern void gc_scav_pair(lispobj where[2]);
 extern void weakobj_init();
@@ -99,14 +103,8 @@ lispobj  copy_large_object(lispobj object, sword_t nwords, int page_type_flag);
 
 lispobj *search_read_only_space(void *pointer);
 lispobj *search_static_space(void *pointer);
-lispobj *search_immobile_space(void *pointer);
 lispobj *search_dynamic_space(void *pointer);
 
-static inline int instruction_ptr_p(void *pointer, lispobj *start_addr)
-{
-    return widetag_of(*start_addr) == CODE_HEADER_WIDETAG &&
-        pointer >= (void*)(start_addr + code_header_words(*start_addr));
-}
 extern int properly_tagged_p_internal(lispobj pointer, lispobj *start_addr);
 static inline int properly_tagged_descriptor_p(void *pointer, lispobj *start_addr) {
   return is_lisp_pointer((lispobj)pointer) &&
@@ -117,6 +115,21 @@ extern void scavenge_control_stack(struct thread *th);
 extern void scrub_control_stack(void);
 extern void scrub_thread_control_stack(struct thread *);
 
+// for code ojects, this bit signifies that this object is in the remembered set.
+// KLUDGE: this constant needs to be autogenerated. It is currently hardcoded into
+// the CODE-HEADER-SET assembly routine for x86 and x86-64.
+#define OBJ_WRITTEN_FLAG 0x40
+#ifdef LISP_FEATURE_LITTLE_ENDIAN
+#define CLEAR_WRITTEN_FLAG(obj) ((unsigned char*)obj)[3] &= ~OBJ_WRITTEN_FLAG
+#define SET_WRITTEN_FLAG(obj)   ((unsigned char*)obj)[3] |= OBJ_WRITTEN_FLAG
+#else
+#define CLEAR_WRITTEN_FLAG(obj) *obj &= ~(OBJ_WRITTEN_FLAG<<24)
+#define SET_WRITTEN_FLAG(obj)   *obj |=  (OBJ_WRITTEN_FLAG<<24)
+#endif
+static inline int header_rememberedp(lispobj header) {
+  return (header & (OBJ_WRITTEN_FLAG << 24)) != 0;
+}
+
 #ifndef LISP_FEATURE_IMMOBILE_SPACE
 
 static inline boolean filler_obj_p(lispobj __attribute__((unused)) *obj) { return 0; }
@@ -124,40 +137,76 @@ static inline boolean filler_obj_p(lispobj __attribute__((unused)) *obj) { retur
 #else
 
 extern void enliven_immobile_obj(lispobj*,int);
-extern void fixup_immobile_refs(lispobj (*)(lispobj), lispobj, struct code*);
 
 #define IMMOBILE_OBJ_VISITED_FLAG    0x10
-#define IMMOBILE_OBJ_GENERATION_MASK 0x0f // mask off the VISITED flag
 
-// Note: this does not work on a SIMPLE-FUN
-// because a simple-fun header does not contain a generation.
-#define __immobile_obj_generation(x) (__immobile_obj_gen_bits(x) & IMMOBILE_OBJ_GENERATION_MASK)
+// Immobile object header word:
+//                 generation byte --|    |-- widetag
+//                                   v    v
+//                       0xzzzzzzzz GGzzzzww
+//         arbitrary data  --------   ---- length in words
+//
+// An an exception to the above, FDEFNs omit the length:
+//                       0xzzzzzzzz zzzzGGww
+//         arbitrary data  -------- ----
+// so that there are 6 consecutive bytes of arbitrary data.
+// The length of an FDEFN is implicitly fixed at 4 words.
+
+// There is a hard constraint on NUM_GENERATIONS, which is currently 8.
+// (0..5=normal, 6=pseudostatic, 7=scratch)
+// Shifting a 1 bit left by the contents of the generation byte
+// must not overflow a register.
+
+// Mask off the VISITED flag to get the generation number
+#define immobile_obj_generation(x) (immobile_obj_gen_bits(x) & 0xf)
 
 #ifdef LISP_FEATURE_LITTLE_ENDIAN
-static inline int immobile_obj_gen_bits(lispobj* pointer) // native pointer
+// Return the generation bits which means the generation number
+// in the 4 low bits (there's 1 excess bit) and the VISITED flag.
+static inline int immobile_obj_gen_bits(lispobj* obj) // native pointer
 {
-  if (widetag_of(*pointer) == SIMPLE_FUN_WIDETAG)
-    pointer = fun_code_header(pointer);
-  return ((generation_index_t*)pointer)[3] & 0x3F;
+    // When debugging, assert that we're called only on a headered object
+    // whose header contains a generation byte.
+    gc_dcheck(!embedded_obj_p(widetag_of(obj)));
+    char gen;
+    switch (widetag_of(obj)) {
+    default:
+        gen = ((generation_index_t*)obj)[3]; break;
+    case FDEFN_WIDETAG:
+        gen = ((generation_index_t*)obj)[1]; break;
+    }
+    return gen & 0x1F;
 }
-// Faster way when we know that the object can't be a simple-fun,
-// such as when walking the immobile space.
-static inline int __immobile_obj_gen_bits(lispobj* pointer) // native pointer
+// Turn a grey node black.
+static inline void set_visited(lispobj* obj)
 {
-  return ((generation_index_t*)pointer)[3] & 0x3F;
+    gc_dcheck(widetag_of(obj) != SIMPLE_FUN_WIDETAG);
+    gc_dcheck(immobile_obj_gen_bits(obj) == new_space);
+    int byte = widetag_of(obj) == FDEFN_WIDETAG ? 1 : 3;
+    ((generation_index_t*)obj)[byte] |= IMMOBILE_OBJ_VISITED_FLAG;
+}
+static inline void assign_generation(lispobj* obj, generation_index_t gen)
+{
+    gc_dcheck(widetag_of(obj) != SIMPLE_FUN_WIDETAG);
+    int byte = widetag_of(obj) == FDEFN_WIDETAG ? 1 : 3;
+    generation_index_t* ptr = (generation_index_t*)obj + byte;
+    // Clear the VISITED flag, assign a new generation, preserving the three
+    // high bits which include the OBJ_WRITTEN flag as well as two
+    // opaque flag bits for use by Lisp.
+    *ptr = (*ptr & 0xE0) | gen;
 }
 #else
 #error "Need to define immobile_obj_gen_bits() for big-endian"
 #endif /* little-endian */
 
 static inline boolean filler_obj_p(lispobj* obj) {
-  return *obj == CODE_HEADER_WIDETAG;
+    return widetag_of(obj) == CODE_HEADER_WIDETAG && obj[1] == 0;
 }
 
 #endif /* immobile space */
 
-#define WEAK_POINTER_NWORDS \
-        ALIGN_UP((sizeof(struct weak_pointer) / sizeof(lispobj)), 2)
+#define WEAK_POINTER_CHAIN_END (void*)(intptr_t)-1
+#define WEAK_POINTER_NWORDS ALIGN_UP(WEAK_POINTER_SIZE,2)
 
 static inline boolean weak_pointer_breakable_p(struct weak_pointer *wp)
 {
@@ -167,11 +216,25 @@ static inline boolean weak_pointer_breakable_p(struct weak_pointer *wp)
     return is_lisp_pointer(pointee) && (from_space_p(pointee)
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
          || (immobile_space_p(pointee) &&
-             immobile_obj_gen_bits(native_pointer(pointee)) == from_space)
+             immobile_obj_gen_bits(base_pointer(pointee)) == from_space)
 #endif
             );
 }
 
+static inline void add_to_weak_pointer_chain(struct weak_pointer *wp) {
+    /* Link 'wp' into weak_pointer_chain using its 'next' field.
+     * We ensure that 'next' is always NULL when the weak pointer isn't
+     * in the chain, and not NULL otherwise. The end of the chain
+     * is denoted by WEAK_POINTER_CHAIN_END which is distinct from NULL.
+     * The test of whether the weak pointer has been placed in the chain
+     * is performed in 'scav_weak_pointer' for gencgc.
+     * In cheneygc, chaining is performed in 'trans_weak_pointer'
+     * which works just as well, since an object is transported
+     * at most once per GC cycle */
+    wp->next = (struct weak_pointer *)LOW_WORD(weak_pointer_chain);
+    weak_pointer_chain = wp;
+}
+
 /// Same as Lisp LOGBITP, except no negative bignums allowed.
 static inline boolean layout_bitmap_logbitp(int index, lispobj bitmap)
 {
@@ -183,8 +246,8 @@ static inline boolean layout_bitmap_logbitp(int index, lispobj bitmap)
 }
 
 /* Keep in sync with 'target-hash-table.lisp' */
-#define hashtable_weakp(ht) (ht->flags & (1<<N_FIXNUM_TAG_BITS))
-#define hashtable_weakness(ht) (ht->flags >> (3+N_FIXNUM_TAG_BITS))
+#define hashtable_weakp(ht) (ht->flags & (8<<N_FIXNUM_TAG_BITS))
+#define hashtable_weakness(ht) (ht->flags >> (4+N_FIXNUM_TAG_BITS))
 
 #if defined(LISP_FEATURE_GENCGC)
 
@@ -258,22 +321,13 @@ static inline void protect_page(void* page_addr, page_index_t page_index)
 
 #endif
 
-#define OBJ_WRITTEN_FLAG 0x40
-#ifdef LISP_FEATURE_LITTLE_ENDIAN
-#define CLEAR_WRITTEN_FLAG(obj) ((unsigned char*)obj)[3] &= ~OBJ_WRITTEN_FLAG
-#define SET_WRITTEN_FLAG(obj)   ((unsigned char*)obj)[3] |= OBJ_WRITTEN_FLAG
-#else
-#define CLEAR_WRITTEN_FLAG(obj) *obj &= ~(OBJ_WRITTEN_FLAG<<24)
-#define SET_WRITTEN_FLAG(obj)   *obj |=  (OBJ_WRITTEN_FLAG<<24)
-#endif
-static inline int header_rememberedp(lispobj header) {
-  return (header & (OBJ_WRITTEN_FLAG << 24)) != 0;
-}
-
 #if defined(LISP_FEATURE_X86_64) || defined(LISP_FEATURE_X86)
 # define CODE_PAGES_USE_SOFT_PROTECTION 1
 #else
 # define CODE_PAGES_USE_SOFT_PROTECTION 0
 #endif
 
+#define KV_PAIRS_HIGH_WATER_MARK(kvv) fixnum_value(kvv[0])
+#define KV_PAIRS_REHASH(kvv) kvv[1]
+
 #endif /* _GC_PRIVATE_H_ */
