@@ -69,9 +69,7 @@ os_zero(os_vm_address_t addr, os_vm_size_t length)
         addr = os_validate(NOT_MOVABLE, block_start, block_size);
 
         if (addr == NULL || addr != block_start)
-            lose("os_zero: block moved! 0x%08x ==> 0x%08x\n",
-                 block_start,
-                 addr);
+            lose("os_zero: block moved! %p ==> %p", block_start, addr);
     }
 }
 #endif
@@ -191,9 +189,29 @@ void os_link_runtime()
 
         link_target += LINKAGE_TABLE_ENTRY_SIZE;
     }
+#ifdef LISP_FEATURE_ARM
+    lispobj* static_space = (lispobj*)STATIC_SPACE_START;
+    extern uword_t alloc_tramp(uword_t);
+    static_space[6] = (lispobj)&alloc_tramp; // for LR rN, [NULL, #k] ; BLX rN
+#endif
+#ifdef CALL_INTO_C
+    extern long call_into_c();
+    SYMBOL(CALL_INTO_C)->value = (lispobj)call_into_c;
+#endif
 #endif /* LISP_FEATURE_SB_DYNAMIC_CORE */
 }
 
+void os_unlink_runtime()
+{
+#ifdef CALL_INTO_C
+    SYMBOL(CALL_INTO_C)->value = UNBOUND_MARKER_WIDETAG;
+#endif
+#ifdef LISP_FEATURE_ARM
+    lispobj* static_space = (lispobj*)STATIC_SPACE_START;
+    static_space[6] = 0;
+#endif
+}
+
 boolean
 gc_managed_heap_space_p(lispobj addr)
 {
@@ -216,17 +234,40 @@ gc_managed_heap_space_p(lispobj addr)
 
 #ifndef LISP_FEATURE_WIN32
 
-/* Remap a part of an already existing mapping to a file */
-void os_map(int fd, int offset, os_vm_address_t addr, os_vm_size_t len)
+/* Remap a part of an already existing memory mapping from a file,
+ * and/or create a new mapping as need be */
+void* load_core_bytes(int fd, int offset, os_vm_address_t addr, os_vm_size_t len)
 {
+    int fail = 0;
+#ifdef LISP_FEATURE_HPUX
+    // Revision afcfb8b5da said that mmap() didn't work on HPUX, changing to use read() instead.
+    // Strangely it also read 4K at a time into a buffer and used memcpy to transfer the buffer.
+    // I don't see why, and given the lack of explanation, I've simplified to 1 read.
+    fail = lseek(fd, offset, SEEK_SET) == (off_t)-1 || read(fd, addr, len) != (ssize_t)len;
+    // This looks bogus but harmlesss, so I'm leaving it.
+    os_flush_icache(addr, len);
+#else
     os_vm_address_t actual;
-
-    actual = mmap(addr, len, OS_VM_PROT_ALL, MAP_PRIVATE | MAP_FIXED,
+    actual = mmap(addr, len,
+                  // If mapping to a random address, then the assumption is
+                  // that we're not going to execute the core; nor should we write to it.
+                  // However, the addr=0 case is for 'editcore' which unfortunately _does_
+                  // write the memory. I'd prefer that it not,
+                  // but that's not the concern here.
+                  addr ? OS_VM_PROT_ALL : OS_VM_PROT_READ | OS_VM_PROT_WRITE,
+                  // Do not pass MAP_FIXED with addr of 0, because most OSes disallow that.
+                  MAP_PRIVATE | (addr ? MAP_FIXED : 0),
                   fd, (off_t) offset);
-    if (actual == MAP_FAILED || (addr && (addr != actual))) {
+    if (actual == MAP_FAILED) {
         perror("mmap");
-        lose("unexpected mmap(%d, %d, ...) failure\n", addr, len);
+        fail = 1;
+    } else if (addr && (addr != actual)) {
+        fail = 1;
     }
+#endif
+    if (fail)
+        lose("load_core_bytes(%d,%x,%lx,%x) failed", fd, offset, (long)addr, (int)len);
+    return (void*)actual;
 }
 
 boolean
