@@ -178,3 +178,12 @@ os_flush_icache(os_vm_address_t address, os_vm_size_t length)
     ppc_flush_icache(address,length);
 }
 
+// "cc -S" on this file shows that the C compiler is responsible for making
+// a substitution for these functions with an additional argument in front.
+// I don't want to know how to do that from Lisp.
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+int _stat(const char *pathname, struct stat *sb) {return stat(pathname, sb); }
+int _lstat(const char *pathname, struct stat *sb) { return lstat(pathname, sb); }
+int _fstat(int fd, struct stat *sb) { return fstat(fd, sb); }
