@@ -24,13 +24,11 @@
 #include "validate.h"
 #include "interr.h"                     /* for declaration of lose */
 
-#if defined(LISP_FEATURE_RELOCATABLE_HEAP)
 #ifdef LISP_FEATURE_CHENEYGC
 uword_t DYNAMIC_0_SPACE_START, DYNAMIC_1_SPACE_START;
 #else
 uword_t DYNAMIC_SPACE_START;
 #endif
-#endif
 
 uword_t asm_routines_start, asm_routines_end;
 
@@ -68,12 +66,24 @@ os_vm_address_t undefined_alien_address = 0;
 
 static void
 ensure_undefined_alien(void) {
-    os_vm_address_t start = os_allocate(os_vm_page_size);
+    os_vm_address_t start = os_validate(MOVABLE|IS_GUARD_PAGE, NULL,
+#ifndef LISP_FEATURE_WIN32
+    /* We can/should disregard our 'os_vm_page_size' constant which tends to be
+     * larger than the granularity that the OS will allow you to manipulate via
+     * mprotect(). e.g. on x86-64-linux we use a page size of 32K but in reality
+     * the protection granularity is 4K.
+     * Moreover, since the memory protection is not changed after allocation,
+     * the granuarity that mprotect() operates on is immaterial. As such, it
+     * probably would work to put N_WORD_BYTES here since that's all we need. */
+                                        getpagesize()
+#else // Use the same value as does contrib/sb-posix/interface.lisp
+                                        4096
+#endif
+                                        );
     if (start) {
-        os_protect(start, os_vm_page_size, OS_VM_PROT_NONE);
         undefined_alien_address = start;
     } else {
-        lose("could not allocate guard page for undefined alien\n");
+        lose("could not allocate guard page for undefined alien");
     }
 }
 
@@ -90,6 +100,13 @@ boolean allocate_hardwired_spaces(boolean hard_failp)
       { READ_ONLY_SPACE_START, READ_ONLY_SPACE_SIZE },
       { STATIC_SPACE_START, STATIC_SPACE_SIZE },
 #ifdef LISP_FEATURE_LINKAGE_TABLE
+      // TODO: if the linkage table were placed just below static space (above
+      // readonly space) and linkage entries were allocated from the end of the
+      // space downward, then a good number of entries would be accessible relative
+      // to NIL which could avoid some extra instructions. This is relevant on
+      // load/store architectures where all calls to C are indirected through a
+      // call_into_c trampoline that first entails loading the address of
+      // that trampoline and then the address of the real callee.
       { LINKAGE_TABLE_SPACE_START, LINKAGE_TABLE_SPACE_SIZE },
 #endif
     };
@@ -116,17 +133,6 @@ boolean allocate_hardwired_spaces(boolean hard_failp)
 void
 allocate_lisp_dynamic_space(boolean did_preinit)
 {
-#ifndef LISP_FEATURE_RELOCATABLE_HEAP
-    // Allocate the largest space(s) first,
-    // since if that fails, it's game over.
-#ifdef LISP_FEATURE_GENCGC
-    ensure_space(NOT_MOVABLE, DYNAMIC_SPACE_START  , dynamic_space_size);
-#else
-    ensure_space(NOT_MOVABLE, DYNAMIC_0_SPACE_START, dynamic_space_size);
-    ensure_space(NOT_MOVABLE, DYNAMIC_1_SPACE_START, dynamic_space_size);
-#endif
-#endif
-
     // Small spaces can be allocated after large spaces are.
     // The above code is only utilized when heap relocation is disabled.
     // And when so, failure to allocate dynamic space is fatal.
