@@ -9,7 +9,7 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
-(in-package :cl-user)
+(enable-test-parallelism)
 
 (defvar *format-mode*)
 
@@ -94,3 +94,56 @@
         ()
         `(lambda () (error ,control))
       (() (condition 'simple-error)))))
+
+(with-test (:name :tokenize-curly-brace-nonliteral-empty)
+  (flet ((try (string)
+           (let ((tokens (sb-format::tokenize-control-string string)))
+             (assert (and (= (length tokens) 3)
+                          (string= (second tokens) ""))))))
+    ;; Each of these curly-brace-wrapped expressions needs to preserve
+    ;; the insides as an empty string so that it does not degenerate
+    ;; to the control string "~{~}".
+    (try "~{~
+~}")
+    (try "~{~0|~}")
+    (try "~{~0~~}")
+    (try "~{~0%~}")
+    ;; these are also each three parsed tokens:
+    ;; 0 newlines plus an ignored newline
+    (try "~{~0%~
+~}")
+    ;; 0 newlines, an ignored newline, and 0 tildes
+    (try "~{~0%~
+~0|~0~~}")))
+
+(with-test (:name :nowarn-princ)
+  ;; you should't see optimization notes from compiling format strings.
+  ;; (the FORMATTER macro is a heavy user of PRINC)
+  (checked-compile
+   '(lambda (x) (declare (optimize speed)) (princ x))
+   :allow-notes nil))
+
+(defun format-to-string-stream (thing string-stream)
+  (declare (notinline format))
+  ;; Tokenizing this string will cons about 224 bytes
+  (format string-stream "Test ~D" thing)
+  ;; CLEAR-OUTPUT should work, but doesn't
+  (file-position string-stream 0))
+
+(with-test (:name :cached-tokenized-string
+            :skipped-on :interpreter)
+  (let ((stream (make-string-output-stream)))
+    (ctu:assert-no-consing (format-to-string-stream 45678 stream))))
+
+(with-test (:name :uncached-tokenized-string)
+  (let ((control-string
+         ;; super-smart compiler might see that the result of
+         ;; this call is constantly "hello ~a"
+         (locally (declare (notinline format)) (format nil "hello ~~a"))))
+    (let ((s1 (format nil control-string '(1 2)))
+          (s2
+           (progn
+             (setf (aref control-string 0) #\Y)
+             (format nil control-string '(1 2)))))
+      (assert (string= s1 "hello (1 2)"))
+      (assert (string= s2 "Yello (1 2)")))))
