@@ -670,7 +670,7 @@ sb-vm::(define-vop (cl-user::test)
 (with-test (:name :typep-compiled-with-jump-table)
   ;; Expect to reference the CTYPE layout because %%TYPEP declares its argument
   ;; to be that.  Expect to reference the UNKNOWN-TYPE layout because of an
-  ;; explicit call to UNKNOWN-TYPE-P. I do not know why NUMERIC-TYPE is there.
+  ;; explicit call to UNKNOWN-TYPE-P; same for FUN-DESIGNATOR-TYPE-P.
   (let ((names
           (mapcar (lambda (x)
                     (sb-kernel:classoid-name (sb-kernel:layout-classoid x)))
@@ -678,5 +678,26 @@ sb-vm::(define-vop (cl-user::test)
     (assert (null (set-difference names
                                   '(sb-kernel:ctype
                                     sb-kernel:unknown-type
-                                    sb-kernel:numeric-type
+                                    sb-kernel:fun-designator-type
                                     #-immobile-space null))))))
+
+;; lp#1857861
+(with-test (:name :undecoded-immediate-data)
+  (let ((f (compile nil '(lambda (x) (when (floatp x) 0.0))))
+        (expect (format nil ", ~D" sb-vm:single-float-widetag)))
+     (loop for line in (split-string (with-output-to-string (string)
+                                       (disassemble f :stream string))
+                                     #\newline)
+             thereis (let ((p (search expect line)))
+                       ;; expect no end-of-line comment
+                       (and p (not (find #\; line :start p))))))
+  (let ((f (compile nil '(lambda (x) (sb-ext:process-p x)))))
+     (loop for line in (split-string (with-output-to-string (string)
+                                       (disassemble f :stream string))
+                                     #\newline)
+             thereis (and (search "LAYOUT for" line)
+                          (search "CMP DWORD PTR" line)))))
+
+(with-test (:name :thread-local-unbound)
+  (let ((c (nth-value 1 (ignore-errors sb-c::*compilation*))))
+    (assert (eq (cell-error-name c) 'sb-c::*compilation*))))
